<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FrontendModelFriend extends JModel
{
  // Tasks
  function accept()
  {
    // Check friends limit
    $friends =& JModel::getInstance('friends', 'FrontendModel');
    if ($friends->friendsLimitReached())
    {
      $settings = new LovefactorySettings();

      if (!$settings->invalid_membership_action)
      {
        JError::raiseWarning(0, JText::_('FRIENDS_TASK_ACCEPT_ERROR_NO_MORE_FRIENDS'));
      }
      $this->_code = 2;

      return false;
    }

    $user =& JFactory::getUser();
    $id   =  JRequest::getVar('id', 0, 'GET', 'integer');

    $friend =& $this->getTable('friend');
    $friend->load($id);

    if (($friend->receiver_id == $user->id) &&
        ($friend->pending == 1) && ($friend->type == 1))
    {
      $friend->accept();

      $settings = new LovefactorySettings();
      if ($settings->enable_messages)
      {
        $message =& $this->getTable('LoveFactoryMessage');
        $message->sendSystemMessage(
          $friend->sender_id,
          JText::_('FRIENDS_TASK_ACCEPT_MESSAGE_TITLE'),
          JText::sprintf('FRIENDS_TASK_ACCEPT_MESSAGE_TEXT', $user->username)
        );
      }

      return true;
    }

    JError::raiseWarning(0, JText::_('FRIENDS_TASK_ACCEPT_ERROR_NOT_FOUND'));
    return false;
  }

  function reject()
  {
    $user =& JFactory::getUser();
    $id   =  JRequest::getVar('id', 0, 'GET', 'integer');

    $friend =& $this->getTable('friend');
    $friend->load($id);

    if (($friend->receiver_id == $user->id) &&
        ($friend->pending == 1) && ($friend->type == 1))
    {
      $friend->delete();

      $settings = new LovefactorySettings();
      if ($settings->enable_messages)
      {
        $message =& $this->getTable('LoveFactoryMessage');
        $message->sendSystemMessage(
          $friend->sender_id,
          JText::_('FRIENDS_TASK_REJECT_MESSAGE_TITLE'),
          JText::sprintf('FRIENDS_TASK_REJECT_MESSAGE_TEXT', $user->username)
        );
      }

      return true;
    }

    JError::raiseWarning(0, JText::_('FRIENDS_TASK_REJECT_ERROR_NOT_FOUND'));
    return false;
  }

  function cancel()
  {
    $user =& JFactory::getUser();
    $id   =  JRequest::getVar('id', 0, 'REQUEST', 'integer');

    $friend =& $this->getTable('friend');
    $friend->load($id);

    if (($friend->sender_id == $user->id) &&
        ($friend->pending == 1) && ($friend->type == 1))
    {
      $friend->delete();

      return true;
    }

    $this->setError(JText::_('FRIENDS_TASK_CANCEL_ERROR_NOT_FOUND'));
    return false;
  }

  function request()
  {
    // Check friends limit
    $friends =& JModel::getInstance('friends', 'FrontendModel');
    if ($friends->friendsLimitReached())
    {
      $this->setError(JText::_('FRIENDS_TASK_REQUEST_ERROR_LIMIT_REACHED'));

      $settings = new LovefactorySettings();
      if ($settings->invalid_membership_action)
      {
        $this->_code = 2;
      }

      return false;
    }

    $id   =  JRequest::getVar('id', 0, 'REQUEST', 'integer');
    $user =& JFactory::getUser();

    // Check if sending request to self
    if ($id == $user->id)
    {
      $this->setError(JText::_('FRIENDS_TASK_REQUEST_ERROR_REQUEST_TO_SELF'));
      return false;
    }

    $query = ' SELECT id'
           . ' FROM #__lovefactory_friends'
           . ' WHERE ((sender_id = ' . $id . ' AND receiver_id = ' . $user->id . ')'
           . ' OR (sender_id = ' . $user->id . ' AND receiver_id = ' . $id . '))'
           . ' AND type = 1';
    $this->_db->setQuery($query);
    $result = $this->_db->loadResult();

    if ($result)
    {
      $this->setError(JText::_('FRIENDS_TASK_REQUEST_ERROR_REQUEST_ALREADY_SENT'));
      return false;
    }

    $message = JRequest::getVar('message', '', 'POST', 'string');

    $friend =& $this->getTable('friend');
    $friend->request($user->id, $id, $message);

    $this->_request_id  = $this->_db->insertid();

    return true;
  }

  function remove()
  {
    $user =& JFactory::getUser();
    $id   =  JRequest::getVar('id', 0, 'REQUEST', 'integer');

    $friend =& $this->getTable('friend');
    $friend->load($id);

    if ((($friend->receiver_id == $user->id) ||
         ($friend->sender_id == $user->id)) &&
        ($friend->pending == 0) && $friend->type == 1)
    {
      $this->_user_id = $friend->sender_id == $user->id ? $friend->receiver_id : $friend->sender_id;
      $friend->remove();

      $settings = new LovefactorySettings();
      if ($settings->enable_messages)
      {
        $message =& $this->getTable('LoveFactoryMessage');
        $message->sendSystemMessage(
          $friend->sender_id == $user->id ? $friend->receiver_id : $friend->sender_id,
          JText::_('FRIENDS_TASK_REMOVE_MESSAGE_TITLE'),
          JText::sprintf('FRIENDS_TASK_REMOVE_MESSAGE_TEXT', $user->username)
        );
      }

      return true;
    }

    $this->setError(JText::_('FRIENDS_TASK_REMOVE_ERROR_NOT_FOUND'));
    return false;
  }

  function addTopFriend()
  {
    $user          =& JFactory::getUser();
    $friendship_id =& JRequest::getVar('friendship', 0, 'GET', 'integer');
    $friend        =& $this->getTable('friend');

    $friend->load($friendship_id);

    // Check if it's my friend
    if ($friend->sender_id != $user->id && $friend->receiver_id != $user->id)
    {
      $this->setError(JText::_('FREINDS_TASK_ADDTOPFRIEND_ERROR_NOT_FOUND'));
      return false;
    }

    // Check if it's already Top Friend
    $status = $friend->sender_id == $user->id ? $friend->sender_status : $friend->receiver_status;
    if ($status == 1)
    {
      $this->setError(JText::_('FRIENDS_TASK_ADDTOPFRIEND_ERROR_ALREADY_TOP_FRIEND'));
      return false;
    }

    $friends =& JModel::getInstance('friends', 'FrontendModel');

    // Check for top friends limit
    if ($friends->friendsLimitReached(1))
    {
      $this->setError(JText::_('FRIENDS_TASK_ADDTOPFRIEND_ERROR_LIMIT_REACHED'));
      $this->_redirectInvalidAction = true;
      return false;
    }

    // All is ok, add Top Friend
    if ($friend->sender_id == $user->id)
    {
      $friend->sender_status = 1;
    }
    else
    {
      $friend->receiver_status = 1;
    }

    // Save friend
    if (!$friend->store())
    {
      $this->setError(JText::_('FRIENDS_TASK_ADDTOPFRIEND_ERROR_DATABASE_ERROR'));
      return false;
    }

    $this->friendship_id = $friendship_id;

    return true;
  }

  function removeTopFriend()
  {
    $user          =& JFactory::getUser();
    $friendship_id =& JRequest::getVar('friendship', 0, 'GET', 'integer');
    $friend        =& $this->getTable('friend');

    $friend->load($friendship_id);

    // Check if it's my friend
    if ($friend->sender_id != $user->id && $friend->receiver_id != $user->id)
    {
      $this->setError(JText::_('FRIENDS_TASK_REMOVETOPFRIEND_ERROR_NOT_FOUND'));
      return false;
    }

    // Check if it's Top Friend
    $status = $friend->sender_id == $user->id ? $friend->sender_status : $friend->receiver_status;
    if ($status == 0)
    {
      $this->setError(JText::_('FRIENDS_TASK_REMOVETOPFRIEND_ERROR_NOT_TOP_FRIEND'));
      return false;
    }

    $friends =& JModel::getInstance('friends', 'FrontendModel');

    // All is ok, remove Top Friend
    if ($friend->sender_id == $user->id)
    {
      $friend->sender_status = 0;
    }
    else
    {
      $friend->receiver_status = 0;
    }

    // Save friend
    if (!$friend->store())
    {
      $this->setError(JText::_('FRIENDS_TASK_REMOVETOPFRIEND_ERROR_DATABASE_ERROR'));
      return false;
    }

    $this->friendship_id = $friendship_id;

    return true;
  }
}
