<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FrontendModelGroupExclude extends JModel
{
  function __construct()
  {
    parent::__construct();

    global $mainframe;

    $this->_limit      =& JRequest::getVar('limit',      3, 'REQUEST', 'integer');
    $this->_limitstart =& JRequest::getVar('limitstart', 0,  'GET',     'integer');

    $this->_limitstart = ($this->_limit != 0 ? (floor($this->_limitstart / $this->_limit) * $this->_limit) : 0);
  }

  function getGroup()
  {
    $user     =& JFactory::getUser();
    $group_id =& JRequest::getVar('group_id', 0, 'GET', 'integer');
    $group    =& $this->getTable('Group', 'Table');

    $group->load($group_id);

    if ($group->user_id != $user->id)
    {
      return false;
    }

    return $group;
  }

  function getData()
  {
    if (empty($this->_data))
    {
      $query       = $this->_buildQuery();
      $this->_data = $this->_getList($query, $this->_limitstart, $this->_limit);

      $this->_processData();
    }

    return $this->_data;
  }

  function getTotal()
  {
    if (empty($this->_total))
    {
      $query        = $this->_buildQuery();
      $this->_total = $this->_getListCount($query);
    }
    return $this->_total;
  }

  function getPagination($ajax = true)
  {
    if (empty($this->_pagination))
    {
      if ($ajax)
      {
        $app    =& JFactory::getApplication();
        $router =& $app->getRouter();

        $router->setVar('format', 'raw');
      }

      jimport('joomla.html.pagination');
      $this->_pagination = new JPagination($this->getTotal(), $this->_limitstart, $this->_limit);
    }

    return $this->_pagination;
  }

  function getGroupsSort()
  {
    $array = array();

    $array[] = JHTML::_('select.option', 0, JText::_('GROUPEXCLUDE_LIST_SORT_USERNAME'));
    $array[] = JHTML::_('select.option', 1, JText::_('GROUPEXCLUDE_LIST_SORT_EXCLUDED_ON'));

    $select = JHTML::_('select.genericlist', $array, 'sort_groups', '', 'value', 'text', $this->getSortByValue());
    $label  = '<label for="sort_groups">' . JText::_('GROUPEXCLUDE_LIST_SORT_LABEL') . ':</label>';

    $selected_order = $this->getSortOrderValue();
    $order  = '<a href="#" id="sort_order" class="lovefactory-button lovefactory-' . ($selected_order ? 'desc' : 'asc') . '" title="' . ($selected_order ? 'desc' : 'asc') . '">' . ($selected_order ? JText::_('GROUPEXCLUDE_LIST_SORT_ORDER_DESC') : JText::_('GROUPEXCLUDE_LIST_SORT_ORDER_ASC')) . '</a>';

    return $label . $select . $order;
  }

  function getSortByValue()
  {
    return JRequest::getVar('lf-groupexclude-sortby', 0, 'COOKIE', 'integer');
  }

  function getSortOrderValue()
  {
    return JRequest::getVar('lf-groupexclude-sortorder', 0, 'COOKIE', 'integer');
  }

  function _buildQuery()
  {
    $user =& JFactory::getUser();

    $where = $this->_buildWhereCondition();
    $order = $this->_buildOrderCondition();

    $query = ' SELECT gb.id, gb.user_id, gb.description, gb.created_at, p.main_photo, u.username,'
           . '   IF(f.id IS NULL, 0, 1) AS is_friend'
           . ' FROM #__lovefactory_group_bans gb'
           . ' LEFT JOIN #__lovefactory_profiles p ON p.user_id = gb.user_id'
           . ' LEFT JOIN #__users u ON u.id = gb.user_id'
           . ' LEFT JOIN #__lovefactory_friends f ON (((f.sender_id = gb.user_id AND f.receiver_id = '.$user->id.') OR (f.receiver_id = gb.user_id AND f.sender_id = '.$user->id.')) AND f.pending = 0)'
           . $where
           . ' GROUP BY gb.user_id'
           . $order;

    return $query;
  }

  function _buildWhereCondition()
  {
    $group_id =& JRequest::getVar('group_id', 0, 'GET', 'integer');

    $where = ' WHERE gb.group_id = ' . $group_id;

    return $where;
  }

  function _buildOrderCondition()
  {
    $sortby    = $this->getSortByValue();
    $sortorder = $this->getSortOrderValue();

    $orderby = ' ORDER BY';

    switch ($sortby)
    {
      case 0: // Username
      default:
        $orderby .= ' u.username';
      break;

      case 1: // Excluded on
        $orderby .= ' gb.created_at';
      break;
    }

    return $orderby . ' ' . ($sortorder ? 'DESC' : 'ASC');
  }

  function _processData()
  {
    global $Itemid;

    foreach ($this->_data as $i => $member)
    {
      $profile =& $this->getTable('Profile', 'Table');
      $profile->bind($member);
      $profile->_is_friend = $member->is_friend;

      $this->_data[$i]->profile     = $profile;
      $this->_data[$i]->created_at  = TheFactoryHelper::format_date($member->created_at, 'ago');
      $this->_data[$i]->description = empty($member->description) ? JText::_('GROUPEXCLUDE_LIST_DESCRIPTION_NOT_AVAILABLE') : nl2br($member->description);
    }
  }

  function remove()
  {
    $user       =& JFactory::getUser();
    $exclude_id =& JRequest::getVar('exclude_id', 0, 'GET', 'integer');

    // Get the exclude record
    $query = ' SELECT gb.id, gb.user_id, g.id AS group_id, g.user_id AS group_user_id'
           . ' FROM #__lovefactory_group_bans gb'
           . ' LEFT JOIN #__lovefactory_groups g ON g.id = gb.group_id'
           . ' WHERE gb.id = ' . $exclude_id;
    $this->_db->setQuery($query);
    $exclude = $this->_db->loadObject();

    if (!$exclude)
    {
      $this->setError(JText::_('GROUP_EXCLUDE_TASK_REMOVE_MEMBER_NOT_FOUND'));
      return false;
    }

    // Check if it's my group
    if ($exclude->group_user_id != $user->id)
    {
      $this->setError(JText::_('GROUP_EXCLUDE_TASK_REMOVE_NOT_YOUR_GROUP'));
      return false;
    }

    $this->_group_id = $exclude->group_id;
    $this->_user_id  = $exclude->user_id;

    // Remove the user from the list
    $exclude =& $this->getTable('GroupBan', 'Table');
    $exclude->delete($exclude_id);

    return true;
  }

  function add()
  {
    $user     =& JFactory::getUser();
    $user_id  =& JRequest::getVar('user_id',  0, 'GET', 'integer');
    $group_id =& JRequest::getVar('group_id', 0, 'GET', 'integer');
    $exclude  =& $this->getTable('GroupBan', 'Table');
    $group    =& $this->getTable('Group', 'Table');
    $profile  =& $this->getTable('Profile', 'Table');

    // Check if it's my group
    $group->load($group_id);

    if ($group->user_id != $user->id)
    {
      $this->setError(JText::_('GROUP_EXCLUDE_TASK_ADD_ERROR_NOT_MY_GROUP'));
      return false;
    }

    // Check if user exists
    $profile->load($user_id);

    if (!$profile->validated)
    {
      $this->setError(JText::_('GROUP_EXCLUDE_TASK_ADD_ERROR_USER_NOT_FOUND'));
      return false;
    }

    // Check if user is already excluded
    $query = ' SELECT gb.id'
           . ' FROM #__lovefactory_group_bans gb'
           . ' WHERE gb.user_id = ' . $user_id
           . ' AND gb.group_id = ' . $group_id;
    $this->_db->setQuery($query);
    $result = $this->_db->loadResult();

    if ($result)
    {
      $this->setError(JText::_('GROUP_EXCLUDE_TASK_ADD_ERROR_USER_ALREADY_EXCLUDED'));
      return false;
    }

    // Remove user from the group
    $query = ' DELETE'
           . ' FROM #__lovefactory_group_members'
           . ' WHERE group_id = ' . $group_id
           . ' AND user_id = ' . $user_id;
    $this->_db->setQuery($query);
    $this->_db->query();

    // Ban user
    $exclude->user_id  = $user_id;
    $exclude->group_id = $group_id;

    $exclude->store();

    $this->_exclude_id = $exclude->id;

    return true;
  }
}