<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FrontendModelGroupMembers extends JModel
{
  function __construct()
  {
    parent::__construct();

    global $mainframe;

    $this->_limit      =& JRequest::getVar('limit',      10, 'REQUEST', 'integer');
    $this->_limitstart =& JRequest::getVar('limitstart', 0,  'GET',     'integer');

    $this->_limitstart = ($this->_limit != 0 ? (floor($this->_limitstart / $this->_limit) * $this->_limit) : 0);
  }

  function getGroup()
  {
    $user     =& JFactory::getUser();
    $group_id =& JRequest::getVar('group_id', 0, 'GET', 'integer');
    $group    =& $this->getTable('Group', 'Table');

    $group->load($group_id);

    // Check if group was found
    if (count($group->_errors))
    {
      return false;
    }

    return $group;
  }

  function getData()
  {
    if (empty($this->_data))
    {
      $query       = $this->_buildQuery();
      $this->_data = $this->_getList($query, $this->_limitstart, $this->_limit);

      $this->_processData();
    }

    return $this->_data;
  }

  function getTotal()
  {
    if (empty($this->_total))
    {
      $query        = $this->_buildQuery();
      $this->_total = $this->_getListCount($query);
    }
    return $this->_total;
  }

  function getPagination($ajax = true)
  {
    if (empty($this->_pagination))
    {
      if ($ajax)
      {
        $app    =& JFactory::getApplication();
        $router =& $app->getRouter();

        $router->setVar('format', 'raw');
      }

      jimport('joomla.html.pagination');
      $this->_pagination = new JPagination($this->getTotal(), $this->_limitstart, $this->_limit);
    }

    return $this->_pagination;
  }

  function getGroupsSort()
  {
    $array = array();

    $array[] = JHTML::_('select.option', 0, JText::_('GROUPMEMBERS_LIST_SORT_USERNAME'));
    $array[] = JHTML::_('select.option', 1, JText::_('GROUPMEMBERS_LIST_SORT_POSTS'));
    $array[] = JHTML::_('select.option', 2, JText::_('GROUPMEMBERS_LIST_SORT_ACTIVITY'));

    $select = JHTML::_('select.genericlist', $array, 'sort_groups', '', 'value', 'text', $this->getSortByValue());
    $label  = '<label for="sort_groups">' . JText::_('GROUPMEMBERS_LIST_SORT_LABEL') . ':</label>';

    $selected_order = $this->getSortOrderValue();
    $order  = '<a href="#" id="sort_order" class="lovefactory-button lovefactory-' . ($selected_order ? 'desc' : 'asc') . '" title="' . ($selected_order ? 'desc' : 'asc') . '">' . ($selected_order ? JText::_('GROUPMEMBERS_LIST_SORT_ORDER_DESC') : JText::_('GROUPMEMBERS_LIST_SORT_ORDER_ASC')) . '</a>';

    return $label . $select . $order;
  }

  function getSortByValue()
  {
    return JRequest::getVar('lf-groupmembers-sortby', 0, 'COOKIE', 'integer');
  }

  function getSortOrderValue()
  {
    return JRequest::getVar('lf-groupmembers-sortorder', 0, 'COOKIE', 'integer');
  }

  function _buildQuery()
  {
    $user =& JFactory::getUser();

    $where = $this->_buildWhereCondition();
    $order = $this->_buildOrderCondition();

    $query = ' SELECT gm.user_id, p.main_photo, u.username,'
           . '   COUNT(gp.id) AS posts,'
           . '   MAX(gp.created_at) AS last_activity,'
           . '   IF(f.id IS NULL, 0, 1) AS is_friend,'
           . '   IF(g.user_id = gm.user_id, 1, 0) AS group_creator'
           . ' FROM #__lovefactory_group_members gm'
           . ' LEFT JOIN #__lovefactory_groups g ON g.id = gm.group_id'
           . ' LEFT JOIN #__lovefactory_profiles p ON p.user_id = gm.user_id'
           . ' LEFT JOIN #__users u ON u.id = gm.user_id'
           . ' LEFT JOIN #__lovefactory_group_posts gp ON gp.group_id = gm.group_id AND gp.user_id = gm.user_id'
           . ' LEFT JOIN #__lovefactory_friends f ON (((f.sender_id = gm.user_id AND f.receiver_id = '.$user->id.') OR (f.receiver_id = gm.user_id AND f.sender_id = '.$user->id.')) AND f.pending = 0)'
           . $where
           . ' GROUP BY gm.user_id'
           . $order;

    return $query;
  }

  function _buildWhereCondition()
  {
    $group_id =& JRequest::getVar('group_id', 0, 'GET', 'integer');

    $where = ' WHERE gm.group_id = ' . $group_id;

    return $where;
  }

  function _buildOrderCondition()
  {
    $sortby    = $this->getSortByValue();
    $sortorder = $this->getSortOrderValue();

    $orderby = ' ORDER BY';

    switch ($sortby)
    {
      case 0: // Username
      default:
        $orderby .= ' u.username';
      break;

      case 1: // Posts
        $orderby .= ' COUNT(gp.id)';
      break;

      case 2: // Last activity
        $orderby .= ' MAX(gp.created_at)';
      break;
    }

    return $orderby . ' ' . ($sortorder ? 'DESC' : 'ASC');
  }

  function _processData()
  {
    global $Itemid;

    foreach ($this->_data as $i => $member)
    {
      $profile =& $this->getTable('Profile', 'Table');
      $profile->bind($member);
      $profile->_is_friend = $member->is_friend;

      $this->_data[$i]->profile       = $profile;
      $this->_data[$i]->last_activity = is_null($member->last_activity) ? JText::_('GROUPS_ACTIVITY_NEVER') : TheFactoryHelper::format_date($member->last_activity, 'ago');
    }
  }
}