<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FrontendModelGroups extends JModel
{
  function __construct()
  {
    parent::__construct();

    global $mainframe;

    $settings = new LoveFactorySettings();

    $this->_limit      = $settings->groups_list_limit;
    $this->_limitstart =& JRequest::getVar('limitstart', 0, 'GET', 'integer');

    $this->_limitstart = ($this->_limit != 0 ? (floor($this->_limitstart / $this->_limit) * $this->_limit) : 0);
  }

  function getData()
  {
    if (empty($this->_data))
    {
      $query       = $this->_buildQuery();
      $this->_data = $this->_getList($query, $this->_limitstart, $this->_limit);

      $this->_processData();
    }

    return $this->_data;
  }

  function getPagination($ajax = true)
  {
    if (empty($this->_pagination))
    {
      if ($ajax)
      {
        $app    =& JFactory::getApplication();
        $router =& $app->getRouter();

        $router->setVar('format', 'raw');
      }

      jimport('joomla.html.pagination');
      $this->_pagination = new JPagination($this->getTotal(), $this->_limitstart, $this->_limit);
    }

    return $this->_pagination;
  }

  function getTotal()
  {
    if (empty($this->_total))
    {
      $query        = $this->_buildQuery();
      $this->_total = $this->_getListCount($query);
    }
    return $this->_total;
  }

  function getGroupsFilter()
  {
    $array = array();

    $array[] = JHTML::_('select.option', 0, JText::_('GROUPS_LIST_FILTER_ALL'));
    $array[] = JHTML::_('select.option', 1, JText::_('GROUPS_LIST_FILTER_CREATED'));
    $array[] = JHTML::_('select.option', 2, JText::_('GROUPS_LIST_FILTER_MEMBER'));

    $select = JHTML::_('select.genericlist', $array, 'filter_groups', '', 'value', 'text', $this->getFilterValue());
    $label  = '<label for="filter_groups">' . JText::_('GROUPS_LIST_FILTER_LABEL') . ':</label>';

    return $label . $select;
  }

  function getGroupsSort()
  {
    $array = array();

    $array[] = JHTML::_('select.option', 0, JText::_('GROUPS_LIST_SORT_NAME'));
    $array[] = JHTML::_('select.option', 1, JText::_('GROUPS_LIST_SORT_MEMBERS'));
    $array[] = JHTML::_('select.option', 2, JText::_('GROUPS_LIST_SORT_ACTIVITY'));
    $array[] = JHTML::_('select.option', 3, JText::_('GROUPS_LIST_SORT_PUBLIC'));

    $select = JHTML::_('select.genericlist', $array, 'sort_groups', '', 'value', 'text', $this->getSortByValue());
    $label  = '<label for="sort_groups">' . JText::_('GROUPS_LIST_SORT_LABEL') . ':</label>';

    $selected_order = $this->getSortOrderValue();
    $order  = '<a href="#" id="sort_order" class="lovefactory-button lovefactory-' . ($selected_order ? 'desc' : 'asc') . '" title="' . ($selected_order ? 'desc' : 'asc') . '">' . ($selected_order ? JText::_('GROUPS_LIST_SORT_ORDER_DESC') : JText::_('GROUPS_LIST_SORT_ORDER_ASC')) . '</a>';

    return $label . $select . $order;
  }

  function getFilterValue()
  {
    return JRequest::getVar('lf-groups-filter-groups', 0, 'COOKIE', 'integer');
  }

  function getSortByValue()
  {
    return JRequest::getVar('lf-groups-sortby-groups', 0, 'COOKIE', 'integer');
  }

  function getSortOrderValue()
  {
    return JRequest::getVar('lf-groups-sortorder-groups', 0, 'COOKIE', 'integer');
  }

  function _buildQuery()
  {
    $user =& JFactory::getUser();

    $where = $this->_buildWhereCondition();
    $order = $this->_buildOrderCondition();

    $query = ' SELECT g.*, u.username,'
           . '   COUNT(DISTINCT m.id) AS members, MAX(p.created_at) AS last_activity,'
           . '   IF(gm.id IS NULL, 0, gm.id) AS is_member,'
           . '   COUNT(DISTINCT p.id) AS posts'
           . ' FROM #__lovefactory_groups g'
           . ' LEFT JOIN #__users u ON u.id = g.user_id'
           . ' LEFT JOIN #__lovefactory_group_members m ON m.group_id = g.id'
           . ' LEFT JOIN #__lovefactory_group_members gm ON gm.group_id = g.id AND gm.user_id = ' . $user->id
           . ' LEFT JOIN #__lovefactory_group_posts p ON p.group_id = g.id'
           . $where
           . ' GROUP BY g.id'
           . $order;

    return $query;
  }

  function _buildWhereCondition()
  {
    $user   =& JFactory::getUser();
    $filter =  $this->getFilterValue();

    $where = ' WHERE 1';

    switch ($filter)
    {
      case 1: // Created by me
        $where .= ' AND g.user_id = ' . $user->id;
      break;

      case 2: // Member of
        $where .= ' AND (gm.id IS NOT NULL AND g.user_id <> ' . $user->id . ')';
      break;
    }

    return $where;
  }

  function _buildOrderCondition()
  {
    $sortby    = $this->getSortByValue();
    $sortorder = $this->getSortOrderValue();

    $orderby = ' ORDER BY';

    switch ($sortby)
    {
      case 0: // Name
      default:
        $orderby .= ' g.title';
      break;

      case 1: // Members
        $orderby .= ' COUNT(m.id)';
      break;

      case 2: // Last activity
        $orderby .= ' MAX(p.created_at)';
      break;

      case 3: // Public
        $orderby .= ' g.private';
      break;
    }

    return $orderby . ' ' . ($sortorder ? 'DESC' : 'ASC');
  }

  function _processData()
  {
    global $Itemid;

    foreach ($this->_data as $i => $group)
    {
      $table =& $this->getTable('Group', 'Table');
      $table->bind($group);

      $table->username      = $group->username;
      $table->members       = $group->members;
      $table->last_activity = is_null($group->last_activity) ? JText::_('GROUPS_ACTIVITY_NEVER') : TheFactoryHelper::format_date($group->last_activity, 'ago');
      $table->description   = '' == $table->description ? JText::_('GROUP_NO_DESCRIPTION') : $table->description;
      $table->is_member     = $group->is_member;
      $table->posts         = $group->posts;
      $table->private_img   = JURI::root() . 'components/com_lovefactory/assets/images/buttons/' . ($group->private ? 'cancel' : 'accept') . '.png';

      $this->_data[$i] = $table;
    }
  }
}