<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class BackendModelBackup extends JModel
{
  function __construct()
  {
    parent::__construct();

    $this->tables = array(
      '#__lovefactory_actions',
      '#__lovefactory_blacklist',
      '#__lovefactory_comments',
      '#__lovefactory_fields',
      '#__lovefactory_friends',
      '#__lovefactory_gateways',
      '#__lovefactory_geo',
      '#__lovefactory_interactions',
      '#__lovefactory_ips',
      '#__lovefactory_memberships',
      '#__lovefactory_memberships_sold',
      '#__lovefactory_messages',
      '#__lovefactory_orders',
      '#__lovefactory_pages',
      '#__lovefactory_pricing',
      '#__lovefactory_payments',
      '#__lovefactory_photos',
      '#__lovefactory_profiles',
      '#__lovefactory_profile_visitors',
      '#__lovefactory_ratings',
      '#__lovefactory_reports',
      '#__lovefactory_statistics_per_day',
      '#__lovefactory_videos',
      '#__lovefactory_video_comments',
      '#__lovefactory_groups',
      '#__lovefactory_group_members',
      '#__lovefactory_group_posts',
      '#__lovefactory_group_bans'
    );
  }

  // Tasks
  function create()
  {
    ini_set('max_execution_time', '120');
    ini_set('memory_limit',       '128M');

    jimport('joomla.filesystem.archive');
    jimport('joomla.filesystem.file');
    jimport('joomla.filesystem.folder');

    $save_settings =& JRequest::getVar('save_settings',    0, 'POST', 'integer');
    $file_name     =  'LoveFactory_Backup_' . date('Y-m-d_His');
    $temp_name     =  md5(time() . rand(0, 100000));
    $temp_folder   =  JPATH_COMPONENT_ADMINISTRATOR.DS.$temp_name;
    $files_to_add  =  array();

    $this->save_photos = 1;

    // Create temp folder
    if (!is_dir($temp_folder))
    {
      mkdir($temp_folder);
    }

    $this->temp_folder = $temp_folder;

    // Check if temp folder is writable
    if (!is_writable($temp_folder))
    {
      $this->cleanup();
      die('Error 1: Temp folder is not writable!');
    }

    // Do we save the settings?
    if ($save_settings)
    {
      JFile::copy(JPATH_COMPONENT_ADMINISTRATOR.DS.'settings.php', $temp_folder.DS.'settings.php');
      $contents = file_get_contents($temp_folder.DS.'settings.php');
      $contents = str_replace('Lovefactory', 'LovefactoryBackup', $contents);

      $settings_file = $temp_folder.DS.'settings.php';

      if (!is_writable($settings_file))
      {
        $this->cleanup();
        die('Error 10: Error saving settings!');
      }

      file_put_contents($settings_file, $contents);

      // Check if the backup settings file exists
      if (!file_exists($settings_file))
      {
        $this->cleanup();
        die('Error 2: Error saving settings!');
      }
    }

    // Create tables
    $this->createTables();

    // Create the info file
    $info = '<?php

$lovefactory_version = "' . $this->getLovefactoryVersion() . '";
$timestamp           = "' . time() . '";
$save_settings       =  ' . $save_settings . ';';
    JFile::write($temp_folder.DS.'info.php', $info);

    // Check if the info file exists
    if (!file_exists($temp_folder.DS.'info.php'))
    {
      $this->cleanup();
      die('Error 4: Error saving info file!');
    }

    // Create the archive
    $files = JFolder::files($temp_folder);

    // Check if there are any files to archive
    if (!count($files))
    {
      $this->cleanup();
      die('Error 5: No files found to archive!');
    }

    foreach ($files as $file)
    {
      $data           = JFile::read($temp_folder.DS.$file);
      $files_to_add[] = array('name' => $file, 'data' => $data);
    }

    $zip =& JArchive::getAdapter('zip');
    $zip->create($temp_folder.DS.$file_name.'.zip', $files_to_add);
    $archive = $temp_folder.DS.$file_name.'.zip';

    // Check if archive file exists
    if (!file_exists($archive))
    {
      $this->cleanup($archive);
      die('Error 6: Error creating archive!');
    }

    // Check if archive file is readable
    if (!is_readable($archive))
    {
      $this->cleanup($archive);
      die('Error 7: Backup archive not readable!');
    }

    // Check if file is an archive
    if (!$zip->checkZipData(file_get_contents($archive)))
    {
      $this->cleanup($archive);
      die('Error 8: Not a valid archive!');
    }

    $filesize = filesize($archive);

    // Send the archive
    header("Pragma: public");
    header("Expires: 0");
    header("Cache-Control: must-revalidate,post-check=0,pre-check=0");
    header("Content-Type: application/zip");
    header('Content-Disposition: attachment; filename="'.$file_name.'.zip'.'"');
    header("Content-Length: ".$filesize);
    header("Content-size: ".$filesize);
    header('Content-Transfer-Encoding: binary');

    @ob_end_clean();
    readfile($archive);
    @ob_start(false);

    // Cleanup
    $this->cleanup($archive);

    exit();
  }

  function restore()
  {
    ini_set('max_execution_time', '120');
    ini_set('memory_limit',       '128M');

    $file   =& JRequest::getVar('backup_file', array(), 'FILES', 'array');
    $append =& JRequest::getVar('append_data', 0,       'POST',  'integer');

    // Check if file uploaded
    if ($file['error'] == 4)
    {
      $this->setError(JText::_('No file uploaded!'));
      return false;
    }

    // Check for errors
    if ($file['error'] || $file['size'] < 1 )
		{
			$this->setError(JText::_('Error code:') . ' ' . $file['error']);
      return false;
		}

    // Extract archive contents
    $temp_name =  md5(time() . rand(0, 100000));
    //$temp_name   =  'temp';
    $temp_folder =  JPATH_COMPONENT_ADMINISTRATOR.DS.$temp_name;

    // Create temp folder
    if (!is_dir($temp_folder))
    {
      mkdir($temp_folder);
    }

    $this->temp_folder = $temp_folder;
    $this->append      = $append;

    jimport('joomla.filesystem.archive');
    jimport('joomla.filesystem.file');

		$zip =& JArchive::getAdapter('zip');
		$zip->extract($file['tmp_name'], $temp_folder);

		// Check for info.php
		if (!file_exists($temp_folder.DS.'info.php'))
		{
		  $this->setError('Not a valid Love Factory Backup File!');
		  return false;
		}

		require_once($temp_folder.DS.'info.php');
		$this->lovefactory_version = intval(str_replace('.', '', $lovefactory_version));

    // Check if the settings are saved
    if (isset($save_settings) && $save_settings == 1 && file_exists($temp_folder.DS.'settings.php'))
    {
      require_once($temp_folder.DS.'settings.php');
      $backup_settings = new LovefactoryBackupSettings();
      $model = JModel::getInstance('settings', 'BackendModel');
      $model->restoreBackup($backup_settings);
    }

    // Restore tables
    if (130 <= $this->lovefactory_version)
    {
      $this->restoreTables();
    }
    else
    {
      $this->restoreTablesOldVersion();
    }

    // Update fields visibility
    if (151 >= $this->lovefactory_version)
    {
      $this->updateFieldsVisibility();
    }

    // Add missing pages
    $this->addPages();

    // Cleanup
    $this->cleanup();

    // Verify users
    $model = JModel::getInstance('user', 'BackendModel');
    $model->verifyUsers(false);

		return true;
  }


  // Create backup
  function createTables()
  {
    foreach ($this->tables as $table)
    {
      $this->createTable($table);
    }
  }

  function createTable($table)
  {
    if ($table == '#__lovefactory_photos' && $this->save_photos == 0)
    {
      return false;
    }

    // Get rows
    $query = ' SELECT t.*'
           . ' FROM ' . $table . ' t';
    $this->_db->setQuery($query);
    $rows = $this->_db->loadAssocList();

    // Check if there is any data
    if (!count($rows))
    {
      return false;
    }

    $file_name = $this->temp_folder.DS.$table.'.txt';
    $handle    = fopen($file_name, 'w');
    $temp      = $this->_db->getTableFields($table);
    $fields    = array();

    // Write header
    foreach ($temp[$table] as $field => $type)
    {
      $fields[] = $field;
    }
    fputcsv($handle, $fields);

    // Write rows
    foreach ($rows as $row)
    {
      $temp = array();
      foreach ($row as $field)
      {
        $temp[] = $this->_db->getEscaped($field);
      }
      fputcsv($handle, $temp);
    }

    fclose($handle);

    // Check if the file exists
    if (!file_exists($file_name))
    {
      $this->cleanup();
      die('Error 3: Error saving table!' . ' ' . $table);
    }
  }


  // Restore backup
  function restoreTables()
  {
    // Restore tables
    foreach ($this->tables as $table)
    {
      $this->restoreTable($table);
    }
  }

  function restoreTable($table)
  {
    $file_name = $this->temp_folder.DS.$table.'.txt';

    // Clear the table
    $this->clearTable($table);

    if (!file_exists($file_name))
    {
      return false;
    }

    $rows      = array();
    $first_row = true;
    $handle    = fopen($file_name, 'r');

    while (($data = fgetcsv($handle, 90000)) !== false)
    {
      if ($first_row)
      {
        $fields = $this->getFirstRow($table, $data);
        $first_row = false;
      }
      else
      {
        $row = $this->getRow($table, $data);
        $this->rowInsert($table, $fields, $row);
      }
    }
    fclose($handle);

    //$this->bulkInsert($table, $fields, $rows);

    if ('#__lovefactory_fields' == $table)
    {
      $fields =& JModel::getInstance('fields', 'BackendModel');
      $fields->verify();
    }
  }

  function updateFieldsVisibility()
  {
    $table = '#__lovefactory_fields_visibility';
    $file_name = $this->temp_folder.DS.$table.'.txt';

    if (!file_exists($file_name))
    {
      return false;
    }

    $rows      = array();
    $first_row = true;
    $handle    = fopen($file_name, 'r');

    while (($data = fgetcsv($handle, 90000)) !== false)
    {
      if ($first_row)
      {
        $fields = $this->getFirstRow($table, $data);
        $first_row = false;
      }
      else
      {
        $row = $this->getRow($table, $data);

        $query = ' UPDATE #__lovefactory_profiles'
               . ' SET field_' . $row[1] . '_visibility = ' . $row[3]
               . ' WHERE user_id = ' . $row[2];
        $this->_db->setQuery($query);
        $this->_db->query();
      }
    }
    fclose($handle);
  }


  // Restore from old version
  function restoreTablesOldVersion()
  {
    // Parse memberships
    $memberships = $this->parseMemberships();

    // Restore fields and pages
    $this->restoreTable('#__lovefactory_fields');
    $this->restoreTable('#__lovefactory_pages');

    // Verify fields integrity
    $fields =& JModel::getInstance('fields', 'BackendModel');
    $fields->verify();

    // Parse profiles
    $profiles = $this->parseProfiles($memberships);

    // Restore rest of tables
    $exclude = array(
      '#__lovefactory_memberships',
      '#__lovefactory_memberships_sold',
      '#__lovefactory_profiles',
      '#__lovefactory_fields',
      '#__lovefactory_pages',
    );

    foreach ($this->tables as $table)
    {
      if (!in_array($table, $exclude))
      {
        $this->restoreTable($table);
      }
    }

    // Add missing pages
    $this->addPages();

    // Clear new tables
    $this->clearTable('#__lovefactory_actions');
    $this->clearTable('#__lovefactory_profile_visitors');
    $this->clearTable('#__lovefactory_orders');
  }

  function parseMemberships()
  {
    $file_name = $this->temp_folder.DS.'#__lovefactory_memberships.txt';

    if (!file_exists($file_name))
    {
      return false;
    }

    $this->clearTable('#__lovefactory_memberships_sold');
    $this->clearTable('#__lovefactory_memberships');

    $memberships = array();
    $rows        = array();
    $first_row   = true;
    $handle      = fopen($file_name, 'r');

    while (($data = fgetcsv($handle, 90000)) !== false)
    {
      if ($first_row)
      {
        $header = $data;
        $fields = array();

        foreach ($data as $value)
        {
          $fields[] = preg_replace("/[^a-zA-Z0-9_\s]/", '', $value);
        }

        $first_row = false;
      }
      else
      {
        $array = array();

        foreach ($data as $i => $value)
        {
          $array[$fields[$i]] = $value;
        }

        $array['new_id'] = null;

        if (1 == $array['default'])
        {
          $table = $this->getTable('membershipsold', 'Table');

	        $table->user_id                  = 0;
	        $table->membership_id            = $array['id'];
	        $table->payment_id               = 0;
	        $table->start_membership         = '0000-00-00 00:00:00';
	        $table->end_membership           = '0000-00-00 00:00:00';
	        $table->expired                  = 0;
	        $table->months                   = 0;
	        $table->title                    = $array['title'];
	        $table->default                  = 1;
	        $table->max_friends              = $array['max_friends'];
	        $table->max_photos               = $array['max_photos'];
	        $table->max_messages_per_day     = $array['max_messages_per_day'];
	        $table->max_interactions_per_day = $array['max_interactions_per_day'];

	        $table->store();

          $array['new_id'] = 1;
        }

        $memberships[$array['id']] = $array;

        $this->rowInsert('#__lovefactory_memberships', $header, $data);

        //$rows[] = $data;
      }
    }
    fclose($handle);

    //$this->clearTable('#__lovefactory_memberships');
    //$this->bulkInsert('#__lovefactory_memberships', $header, $rows);

    return $memberships;
  }

  function parseProfiles($memberships)
  {
    $file_name = $this->temp_folder.DS.'#__lovefactory_profiles.txt';

    if (!file_exists($file_name))
    {
      return false;
    }

    $this->clearTable('#__lovefactory_profiles');

    $rows      = array();
    $first_row = true;
    $handle    = fopen($file_name, 'r');

    while (($data = fgetcsv($handle, 90000)) !== false)
    {
      if ($first_row)
      {
        $header = array();
        foreach ($data as $value)
        {
          $header[] = preg_replace("/[^a-zA-Z0-9_\s]/", '', $value);
        }

        $header[4] = 'membership_sold_id';
        unset($header[5], $header[6]);

        $query = ' SHOW COLUMNS FROM #__lovefactory_profiles';
        $this->_db->setQuery($query);
        $columns = $this->_db->loadObjectList('Field');

        $unset = array();

        foreach ($header as $i => $field)
        {
          if ('looking' == $field)
          {
            $looking_id = $i;
          }

          if ('main_photo' == $field)
          {
            $main_photo_id = $i;
          }

          $exploded = explode('_', $field);

          if ('field' == $exploded[0])
          {
            if (!array_key_exists($field, $columns))
            {
              $unset[] = $i;
            }
          }
        }

        foreach ($unset as $id)
        {
          unset($header[$id]);
        }

        $first_row = false;
      }
      else
      {
        foreach ($unset as $id)
        {
          unset($data[$id]);
        }

        $membership = $memberships[$data[4]];

        if (null != $membership['new_id'])
        {
          $data[4] = $membership['new_id'];
        }
        else
        {
          $query = ' INSERT'
                 . ' INTO #__lovefactory_memberships_sold'
                 . ' VALUES ("", ' . $data[0] . ', ' . $membership['id'] . ', 0, "", "' . $data[5] . '", "' . $data[6] . '", 0, 0, "' . $membership['title'] . '", 0, "' . $membership['max_photos'] . '", "' . $membership['max_friends'] . '", "' . $membership['max_messages_per_day'] . '", "' . $membership['max_interactions_per_day'] . '")';
          $this->_db->setQuery($query);
          $this->_db->query();
          $data[4] = $this->_db->insertid();
        }

        unset($data[5], $data[6]);

        // Prepare Looking for field
        $data[$looking_id] = '*|*' . $data[$looking_id] . '*|*';

        // Prepare main photo
        $data[$main_photo_id] = str_replace('thumb_', '', $data[$main_photo_id]);

        //$rows[] = $data;
        $this->rowInsert('#__lovefactory_profiles', $header, $data);
      }
    }
    fclose($handle);

//    $this->clearTable('#__lovefactory_profiles');
//    $this->bulkInsert('#__lovefactory_profiles', $header, $rows);

    return $memberships;
  }

  function addPages()
  {
    $query = ' SELECT p.type_id'
           . ' FROM #__lovefactory_pages p';
    $this->_db->setQuery($query);
    $pages = $this->_db->loadObjectList('type_id');

    $titles = array(
      7 => 'Friends list view',
      8 => 'Profile Fillin',
      9 => 'Map member info',
    );

    for ($i = 1; $i <= 9; $i++)
    {
      if (!array_key_exists($i, $pages))
      {
        if (!isset($fields_string))
        {
          $query = ' SELECT f.id'
                 . ' FROM #__lovefactory_fields f'
                 . ' WHERE f.type_id = 10';
          $this->_db->setQuery($query);
          $id = $this->_db->loadResult();

          $fields_string = '1_0_' . $id;
        }

        $page =& $this->getTable('page', 'Table');
        $page->type_id = $i;
        $page->title   = $titles[$i];
        $page->fields  = $fields_string;

        $page->store();
      }
    }
  }


  // Helpers
  function getLovefactoryVersion()
  {
    $file = JPATH_COMPONENT_ADMINISTRATOR.DS.'com_lovefactory.xml';
    $data = JApplicationHelper::parseXMLInstallFile($file);

    return $data['version'];
  }

  function getFirstRow($table, $data)
  {
    if (130 <= $this->lovefactory_version)
    {
      return $data;
    }

    switch ($table)
    {
      case '#__lovefactory_payments':
        unset($data[2], $data[4], $data[6]);

        $data[1] = 'received_at';
        $data[5] = 'amount';
        $data[8] = 'gateway';
      break;
    }

    return $data;
  }

  function getRow($table, $data)
  {
     if (130 <= $this->lovefactory_version)
    {
      return $data;
    }

    switch ($table)
    {
      case '#__lovefactory_payments':
        unset($data[2], $data[4], $data[6]);
      break;
    }

    return $data;
  }

  function clearTable($table)
  {
    $query = ' TRUNCATE TABLE ' . $table;
    $this->_db->setQuery($query);
    $this->_db->query();
  }

  function bulkInsert($table, $fields, $rows)
  {
    $temp = array();
    foreach ($rows as $row)
    {
      $temp[] = '\'' . implode('\', \'', $row) . '\'';
    }

    $query = ' INSERT INTO ' . $table . ' (`' . implode('`, `', $fields) . '`) VALUES (' . implode('), (', $temp) . ')';
    $this->_db->setQuery($query);
    $this->_db->query();
  }

  function rowInsert($table, $fields, $row)
  {
    $query = ' INSERT INTO ' . $table . ' (`' . implode('`, `', $fields) . '`) VALUES (\'' . implode('\', \'', $row) . '\')';
    $this->_db->setQuery($query);
    $this->_db->query();
  }

  function cleanup($archive = null)
  {
    // Settings file
    if (file_exists($this->temp_folder.DS.'settings.php'))
    {
      @unlink($this->temp_folder.DS.'settings.php');
    }

    // Info file
    @unlink($this->temp_folder.DS.'info.php');

    // Archive
    if (!is_null($archive))
    {
      @unlink($archive);
    }

    // Tables
    foreach ($this->tables as $table)
    {
      if (file_exists($this->temp_folder.DS.$table.'.txt'))
      {
        @unlink($this->temp_folder.DS.$table.'.txt');
      }
    }

    // Folder
    JFolder::delete($this->temp_folder);
  }
}