<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class BackendModelFields extends JModel
{
  var $_data;
  var $_total;
  var $_pagination;

  function __construct()
  {
    parent::__construct();

    global $mainframe, $option;
    $limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
    $limitstart = $mainframe->getUserStateFromRequest($option.'.limitstart', 'limitstart', 0, 'int');

    $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

    $this->setState('limit',      $limit);
    $this->setState('limitstart', $limitstart);
  }

  function getData()
  {
    if (empty($this->_data))
    {
      $query = $this->_buildQuery();
      $this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));

      $types = $this->getTypes();

      foreach ($this->_data as $i => $field)
      {
        $this->_data[$i]->type_name = isset($types[$field->type_id]) ? ucfirst($types[$field->type_id]) : JText::_('System');
      }
    }

    return $this->_data;
  }

  function _buildQuery()
  {
    $orderby = $this->_buildContentOrderBy();
    $where   = $this->_buildContentWhere();

    $query = ' SELECT *'
           . ' FROM #__lovefactory_fields'
           . $where
           . $orderby;

    return $query;
  }

  function _buildContentOrderBy()
  {
    global $mainframe, $option;

    $filter_order     = $mainframe->getUserStateFromRequest($option . '.filter_order',     'filter_order',     'title', 'cmd');
    $filter_order_Dir = $mainframe->getUserStateFromRequest($option . '.filter_order_Dir', 'filter_order_Dir', 'asc',   'word');

    $orderby = '';

    if (in_array($filter_order, array('title', 'type_id', 'published', 'required', 'system')))
    {
      $orderby = ' ORDER BY ' . $filter_order . ' ' . $filter_order_Dir;
    }

    return $orderby;
  }

  function _buildContentWhere()
  {
    global $mainframe, $option;

    $search = $mainframe->getUserStateFromRequest($option . '.search',        'search',       '', 'cmd');
    $state  = $mainframe->getUserStateFromRequest($option . '.filter_state',  'filter_state', '', 'cmd');
    $system = $mainframe->getUserStateFromRequest($option . '.fields.system', 'system',       '', 'cmd');

    $where = ' WHERE 1 = 1';

    if ($system != '')
    {
      $where .= ' AND system = ' . $system;
    }

    if ($search != '')
    {
      $where .= ' AND title LIKE "%' . $search . '%"';
    }

    if ($state != '')
    {
      $state  = ($state == 'P') ? 1 : 0;
      $where .= ' AND published = ' . $state;
    }

    return $where;
  }

  function getTotal()
  {
    if (empty($this->_total))
    {
      $query = $this->_buildQuery();
      $this->_total = $this->_getListCount($query);
    }

    return $this->_total;
  }

  function getPagination()
  {
    if (empty($this->_pagination))
    {
      jimport('joomla.html.pagination');
      $this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit'));
    }

    return $this->_pagination;
  }


  function getTypes()
  {
    $types = array(
      1  => JText::_('Text field'),
      2  => JText::_('Text area'),
      3  => JText::_('Drop down (single select)'),
      4  => JText::_('Drop down (multiple select)'),
      5  => JText::_('Checkboxes'),
      6  => JText::_('Radio buttons'),
      7  => JText::_('Birth date'),
      8  => JText::_('Membership'),
      9  => JText::_('Main photo'),
      10 => JText::_('Sex'),
      11 => JText::_('Looking for'),
      12 => JText::_('Username'),
      13 => JText::_('Online members (search condition)'),
      14 => JText::_('With pictures (search condition)'),
      15 => JText::_('Last seen'),
      16 => JText::_('No. of photos'),
      17 => JText::_('Online status'),
      18 => JText::_('Email'),
      19 => JText::_('Password'),
      20 => JText::_('Repeat password'),
      21 => JText::_('Main photo upload'),
      22 => JText::_('No. of friends'),
      23 => JText::_('Google map location'),
      24 => JText::_('Distance'),
      25 => JText::_('Relationship'),
      26 => JText::_('ReCaptcha'),
      27 => JText::_('Spacer'),
      28 => JText::_('Terms and Conditions'),
      29 => JText::_('Profile rating'),
    );

    return $types;
  }


  function verify()
  {
    $titles = $this->getTypes();

    // 0. Check if we have system fields that are not marked as system in table
    $field =& $this->getTable('field', 'Table');
    /* @var $field TableField */
    $types = array_keys($field->getSystemTypes());
    $query = ' SELECT f.id, f.type_id'
           . ' FROM #__lovefactory_fields f'
           . ' WHERE f.type_id IN (' . implode(',', $types) . ')'
           . ' AND f.system = 0';
    $this->_db->setQuery($query);
    $fields = $this->_db->loadObjectList();

    foreach ($fields as $field)
    {
      $query = ' UPDATE #__lovefactory_fields'
             . ' SET system = 1'
             . ' WHERE id = ' . $field->id;
      $this->_db->setQuery($query);
      $this->_db->query();

      echo '<p>' . $titles[$field->type_id] . ' - ' . JText::_('Set as System field') . '</p>';
    }

    // 1. Check if system fields exist
    $query = ' SELECT f.id, f.type_id, f.title, f.system'
           . ' FROM #__lovefactory_fields f'
           . ' WHERE f.type_id IN (' . implode(',', $types) . ')';

    $this->_db->setQuery($query);
    $fields = $this->_db->loadObjectList('type_id');

    foreach ($types as $type)
    {
      if (!array_key_exists($type, $fields))
      {
        echo '<p>' . $titles[$type] . ' - ' . JText::_('System field created') . '</p>';
        $field =& $this->getTable('field', 'Table');

        $field->type_id         = $type;
        $field->system          = 1;
        $field->published       = 0;
        $field->visibility      = 1;
        $field->user_visibility = 0;
        $field->title           = $titles[$type];

        $field->store();
      }
    }

    // 2. Check if user fields are in the profile table
    $field =& $this->getTable('field', 'Table');
    $types = array_keys($field->getSystemTypes());

    $query = ' SELECT f.id, f.type_id, f.title'
           . ' FROM #__lovefactory_fields f'
           . ' WHERE f.system = 0'
           . ' OR f.type_id = 7';

    $this->_db->setQuery($query);
    $fields = $this->_db->loadObjectList();

    foreach ($fields as $field)
    {
      $table =& $this->getTable('field', 'Table');
      $table->load($field->id);

      /* @var $table TableField */
      $table->store();
    }

    // 3. Check for orphaned fields in the db
    $query = ' SELECT f.id, CONCAT("field_", f.id) AS title'
           . ' FROM #__lovefactory_fields f'
           . ' WHERE f.system = 0'
           . ' OR f.type_id = 7';
    $this->_db->setQuery($query);
    $user_fields = $this->_db->loadObjectList('title');

    foreach ($user_fields as $field)
    {
      $user_fields[$field->title . '_visibility'] = array();
    }

    $query = ' SHOW COLUMNS FROM #__lovefactory_profiles WHERE Field LIKE "%field_%"';
    $this->_db->setQuery($query);
    $profile_fields = $this->_db->loadObjectList('Field');

    foreach ($profile_fields as $field)
    {
      if (!array_key_exists($field->Field, $user_fields))
      {
        $query = ' ALTER TABLE #__lovefactory_profiles'
               . ' DROP ' . $field->Field;
        $this->_db->setQuery($query);
        $this->_db->query();

        echo '<p>' . $field->Field . ' - ' . JText::_('Orphan field removed from profiles table') . '</p>';
      }
    }

    // 4. Check for orphaned fields in pages
    $query = ' SELECT p.*'
           . ' FROM #__lovefactory_pages p';
    $this->_db->setQuery($query);
    $pages = $this->_db->loadObjectList();

    $query = ' SELECT f.id'
           . ' FROM #__lovefactory_fields f';
    $this->_db->setQuery($query);
    $all_fields = $this->_db->loadObjectList('id');

    foreach ($pages as $page)
    {
      $fields  = explode('#', $page->fields);
      $removed = false;

      foreach ($fields as $i => $field)
      {
        $field = explode('_', $field);
        if (!array_key_exists($field[2], $all_fields))
        {
          unset($fields[$i]);
          $removed = true;

          echo '<p>' . $field[2] . '/' . $page->title . ' - ' . JText::_('Removed orphan field from page') . '</p>';
        }
      }

      if ($removed)
      {
        $table =& $this->getTable('page', 'Table');
        $table->load($page->id);

        $table->fields = implode('#', $fields);
        $table->store();
      }
    }

    echo '<p><b>' . JText::_('Fields integrity check finished!') . '</b></p>';
  }
}