<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class BackendModelMemberships extends JModel
{
  var $_data;
  var $_total;
  var $_pagination;

  function __construct()
  {
    parent::__construct();

    global $mainframe, $option;
    $limit      = $mainframe->getUserStateFromRequest('global.list.limit',               'limit',      $mainframe->getCfg('list_limit'), 'int');
    $limitstart = $mainframe->getUserStateFromRequest($option.'.memberships.limitstart', 'limitstart', 0,                                'int');

    $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

    $this->setState('limit',      $limit);
    $this->setState('limitstart', $limitstart);
  }

  // List
  function getData()
  {
    if (empty($this->_data))
    {
      $query = $this->_buildQuery();
      $this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
    }

    return $this->_data;
  }

  function _buildQuery()
  {
    $orderby = $this->_buildContentOrderBy();
    $where   = $this->_buildContentWhere();

    $query = ' SELECT m.*,'
           . '   IF (m.max_friends=-1, "'              . JText::_('unlimited') . '", m.max_friends)              AS max_friends,'
           . '   IF (m.max_photos=-1, "'               . JText::_('unlimited') . '", m.max_photos)               AS max_photos,'
           . '   IF (m.max_videos=-1, "'               . JText::_('unlimited') . '", m.max_videos)               AS max_videos,'
           . '   IF (m.max_messages_per_day=-1, "'     . JText::_('unlimited') . '", m.max_messages_per_day)     AS max_messages_per_day,'
           . '   IF (m.max_interactions_per_day=-1, "' . JText::_('unlimited') . '", m.max_interactions_per_day) AS max_interactions_per_day,'
           . '   IF (m.top_friends=-1, "'              . JText::_('unlimited') . '", m.top_friends)              AS top_friends,'
           . '   IF (m.groups_create=-1, "'            . JText::_('unlimited') . '", m.groups_create)            AS groups_create,'
           . '   IF (m.groups_join=-1, "'              . JText::_('unlimited') . '", m.groups_join)              AS groups_join,'
           . '   COUNT(p.user_id) AS users'
           . ' FROM #__lovefactory_memberships m'
           . ' LEFT JOIN #__lovefactory_memberships_sold s ON s.membership_id = m.id'
           . ' LEFT JOIN #__lovefactory_profiles p ON p.membership_sold_id = s.id'
           . $where
           . ' GROUP BY m.id'
           . $orderby;

    #var_dump($query);

    return $query;
  }

  function _buildContentOrderBy()
  {
    global $mainframe, $option;

    $filter_order     = $mainframe->getUserStateFromRequest($option.'.memberships.filter_order',     'filter_order',     'title', 'cmd');
    $filter_order_Dir = $mainframe->getUserStateFromRequest($option.'.memberships.filter_order_Dir', 'filter_order_Dir', 'asc',   'word');

    $orderby = '';

    if (in_array($filter_order, array('m.title',      'm.max_friends',
                                      'm.max_photos', 'm.max_messages_per_day',
                                      'm.max_interactions_per_day',
                                      'm.ordering',   'm.published',
                                      'users',        'm.gmaps',
                                      'm.shoutbox',   'm.chatfactory',
                                      'm.id',         'm.max_videos')))
    {
      switch ($filter_order)
      {
        case 'users':
          $filter_order = 'COUNT(p.user_id)';
        break;
      }

      $orderby = ' ORDER BY ' . $filter_order . ' ' . $filter_order_Dir;
    }

    return $orderby;
  }

  function _buildContentWhere()
  {
    global $mainframe, $option;

    $state = $mainframe->getUserStateFromRequest($option.'.memberships.filter_state', 'filter_state', '', 'cmd');

    $where = ' WHERE 1';

    if ($state != '')
    {
      $state  = ($state == 'P') ? 1 : 0;
      $where .= ' AND m.published = ' . $state;
    }

    return $where;
  }

  function getTotal()
  {
    if (empty($this->_total))
    {
      $query = $this->_buildQuery();
      $this->_total = $this->_getListCount($query);
    }

    return $this->_total;
  }

  function getPagination()
  {
    if (empty($this->_pagination))
    {
      jimport('joomla.html.pagination');
      $this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit'));
    }

    return $this->_pagination;
  }

  function checkDefaultSoldMembership()
  {
    $membership =& $this->getTable('membershipsold', 'Table');
    $membership->load(1);

    if (count($membership->_errors))
    {
      $query = ' SELECT m.*'
             . ' FROM #__lovefactory_memberships m'
             . ' WHERE m.default = 1';
      $this->_db->setQuery($query);
      $default = $this->_db->loadObject();

      $membership->title                    = $default->title;
      $membership->membership_id            = $default->id;
      $membership->default                  = 1;
      $membership->max_photos               = $default->max_photos;
      $membership->max_videos               = $default->max_videos;
      $membership->max_friends              = $default->max_friends;
      $membership->max_messages_per_day     = $default->max_messages_per_day;
      $membership->max_interactions_per_day = $default->max_interactions_per_day;
      $membership->shoutbox                 = $default->shoutbox;
	    $membership->chatfactory              = $default->chatfactory;
	    $membership->top_friends              = $default->top_friends;

      $this->_db->insertObject($membership->getTableName(), $membership, 1);
    }
  }

  function getShoutbox()
  {
    $membership =& $this->getTable('membership');

    return $membership->getShoutboxValues();
  }

  function getAccess()
  {
    $array = array(
      'advancedsearch' => 'Advanced search',
      'blacklist'      => 'Ignore list',
      'comment'        => 'Comment (add, delete, report)',
      //'edit'           => 'Profile edit',
      'friend'         => 'Friendship (request, accept, remove, cancel, reject)',
      'friends'        => 'Friends list',
      'friendspending' => 'Pending friendships requests list',
      'gallery'        => 'Gallery',
      'inbox'          => 'Inbox',
      'interaction'    => 'Interaction (send, respond)',
      'interactions'   => 'Interactions list',
      'mailbox'        => 'Mailbox (empty inbox, empty outbox)',
      'membersmap'     => 'Members map',
      'message'        => 'Message (send, delete, report)',
      'messageread'    => 'Read message',
      'messagewrite'   => 'Write message',
      'myfriends'      => 'My friends list',
      'mygallery'      => 'My gallery',
      'online'         => 'Online users list',
      'outbox'         => 'Outbox',
      'otherprofiles'  => 'Other profiles',
      'rating'         => 'Rating (add)',
      'radiussearch'   => 'Radius search',
      'quicksearch'    => 'Quick search',
      'status'         => 'Status',
      'wallpage'       => 'Detailed wallpage',
    );

    return $array;
  }

  // Move users
  function getMembershipsToMove()
  {
    $cid = JRequest::getVar('cid', array(), 'post', 'array');
    JArrayHelper::toInteger($cid);

    $query = ' SELECT m.id, m.title, COUNT(p.user_id) AS users'
           . ' FROM #__lovefactory_memberships m'
           . ' LEFT JOIN #__lovefactory_profiles p ON p.membership = m.id'
           . ' WHERE m.id IN (' . implode( ',', $cid ) . ')'
           . ' GROUP BY m.id';

    $this->_db->setQuery($query);
    return $this->_db->loadObjectList();
  }

  function moveUsers()
  {
    $cid           =& JRequest::getVar('cid',           array(), 'post', 'array');
    $membership_id =& JRequest::getVar('membership_id', 0,       'post', 'integer');
    JArrayHelper::toInteger($cid);

    $query = ' UPDATE #__lovefactory_profiles'
           . ' SET membership = ' . $membership_id
           . ' WHERE membership IN (' . implode(',', $cid) . ')';
    $this->_db->setQuery($query);
    $this->_db->query();

    return true;
  }
}