<?php

defined('_JEXEC') or die('Restricted access');

class TableField extends JTable
{
	var $id              = null;
	var $type_id         = 1;
	var $system          = null;
	var $title           = null;
	var $title_view      = null;
	var $title_edit      = null;
	var $title_search    = null;
	var $show_title      = 1;
	var $required        = 0;
	var $description     = null;
	var $published       = 0;
	var $params          = null;
	var $visibility      = 1;
	var $user_visibility = 1;

	function __construct(&$db)
	{
		parent::__construct('#__lovefactory_fields', 'id', $db);
	}

	function loadArray($array)
	{
	  if (!isset($array[0]) || $array[0] == null)
	  {
	    return false;
	  }

	  $id = JRequest::getVar('id', 0, 'REQUEST', 'integer');

	  $query = ' SELECT f.*'
	         . ' FROM #__lovefactory_fields f'
	         #. ' LEFT JOIN #__lovefactory_fields_visibility v ON f.id = v.field_id AND v.user_id = ' . $id
	         . ' WHERE f.id IN (' . implode(',', $array) . ')';
	  $this->_db->setQuery($query);

	  #var_dump($this->_db->_sql);

	  $fields = $this->_db->loadObjectList();
	  $result = array();

	  foreach ($fields as $field)
	  {
	    $result[$field->id] = $field;
	  }

	  return $result;
	}

	/**
	 * Parses and returns the value from a POST request
	 *
	 * @return unknown
	 */
	function parseValueFromRequest($profile = null, $errors = true)
	{
	  $this->title = '' == $this->title_edit ? $this->title : $this->title_edit;

	  switch ($this->type_id)
	  {
	    case 1: // Text input
	      $request =& JRequest::getVar('field_' . $this->id, '', 'POST', 'string');

	      if ($request == '' && $this->required)
        {
          if ($errors)
          {
            JError::raiseWarning(0, JText::sprintf('FIELD_ERROR_IS_REQUIRED', $this->title));
          }

          return false;
        }

        return $request;
	      break;

	    case 2: // Text area
	      $request =& JRequest::getVar('field_' . $this->id, '', 'POST', 'string', 2);

	      $tags    = explode(',', $this->getParam('allowed_tags'));
	      $request = strip_tags($request, '<' . implode('><', $tags) . '>');

	      if ($request == '' && $this->required)
        {
          if ($errors)
          {
            JError::raiseWarning(0, JText::sprintf('FIELD_ERROR_IS_REQUIRED', $this->title));
          }

          return false;
        }

        return $request;
	      break;

      case 3:  // Drop down (single select)
      case 6:  // Radio buttons
      case 10: // Sex
        if (10 == $this->type_id)
        {
          $settings = new LovefactorySettings();
          if (!$settings->gender_change && null != $profile->sex)
          {
            return null;
          }
        }

        $request  =& JRequest::getVar('field_' . $this->id, -1, 'POST', 'integer');
        $defaults =  $this->getDefaultValues();

        if (!array_key_exists($request, $defaults) && $request != -1)
        {
          if ($errors)
          {
            JError::raiseWarning(0, JText::sprintf('FIELD_ERROR_IS_INVALID', $this->title));
          }
          return false;
        }

        if ($this->required && $request == -1)
        {
          if ($errors)
          {
            JError::raiseWarning(0, JText::sprintf('FIELD_ERROR_IS_REQUIRED', $this->title));
          }
          return false;
        }

        return $request == -1 ? '' : $defaults[$request];
      break;

      case 11: // Looking
        switch ($this->getParam('mode', 1))
        {
          case 1: // Drop down (single select)
          case 2: // Radio buttons
            $request  =& JRequest::getVar('field_' . $this->id, -1, 'POST', 'integer');
            $defaults =  $this->getDefaultValues();

            if (!array_key_exists($request, $defaults) && $request != -1)
            {
              if ($errors)
              {
                JError::raiseWarning(0, JText::sprintf('FIELD_ERROR_IS_INVALID', $this->title));
              }

              return false;
            }

            if ($this->required && $request == -1)
            {
              if ($errors)
              {
                JError::raiseWarning(0, JText::sprintf('FIELD_ERROR_IS_REQUIRED', $this->title));
              }

              return false;
            }

            return $request == -1 ? '' : '*|*' . $defaults[$request] . '*|*';
            break;

          case 3:
          case 4:
            $request = JRequest::getVar('field_' . $this->id, array(), 'POST', 'array');

            if ($this->getParam('mode', 1) == 3)
            {
              $request = array_flip($request);
            }

            $defaults = $this->getDefaultValues();
            $response = array_intersect_key($defaults, $request);

            if (!count($response) && $this->required)
            {
              if ($errors)
              {
                JError::raiseWarning(0, JText::sprintf('FIELD_ERROR_IS_REQUIRED', $this->title));
              }

              return false;
            }

            $response = '*|*' . implode('*|*', $response) . '*|*';

            return $response;
            break;
        }
        $request  =& JRequest::getVar('field_' . $this->id, -1, 'POST', 'integer');
      break;

      case 4: // Drop down (multiple select)
      case 5: // Checkboxes
        $request = JRequest::getVar('field_' . $this->id, array(), 'POST', 'array');

        if ($this->type_id == 4)
        {
          $request = array_flip($request);
        }

        $defaults = $this->getDefaultValues();
        $response = array_intersect_key($defaults, $request);

        if (!count($response) && $this->required)
        {
          if ($errors)
          {
            JError::raiseWarning(0, JText::sprintf('FIELD_ERROR_IS_REQUIRED', $this->title));
          }

          return false;
        }

        $response = '*|*' . implode('*|*', $response) . '*|*';

        return $response;
      break;

      case 7: // Birth date
        $request =& JRequest::getVar('field_' . $this->id, array(), 'POST', 'array');

        if (!checkdate(intval($request['month']), intval($request['day']), intval($request['year'])))
        {
          if ($errors)
          {
            JError::raiseWarning(0, JText::sprintf('FIELD_ERROR_IS_INVALID', $this->title));
          }
          return false;
        }

        $request['day']   = str_pad($request['day'], 2, '0', STR_PAD_LEFT);
        $request['month'] = str_pad($request['month'], 2, '0', STR_PAD_LEFT);

        return implode('', array_reverse($request));
      break;

      case 21: // Main photo upload
        $request     =& JRequest::getVar('main_photo_upload', '', 'POST', 'string');
        $valid_photo =  file_exists(JPATH_COMPONENT_SITE.DS.'storage'.DS.'registration_photos'.DS.$request);

        if ($valid_photo)
        {
          return $request;
        }

        if ($this->required)
        {
          if ($errors)
          {
            JError::raiseWarning('ERROR', JText::_('FIELD_PHOTO_UPLOAD_ERROR'));
          }

          return false;
        }

        return '';
      break;

      case 23: // Google map location
        $lat =& JRequest::getVar('gmap_lat', 0, 'POST', 'float');
        $lng =& JRequest::getVar('gmap_lng', 0, 'POST', 'float');
        $geo =& JTable::getInstance('geo', 'Table');

        $geo->load($profile->user_id);

        $geo->lat = $lat;
        $geo->lng = $lng;

        if (count($geo->_errors))
        {
          $user =& JFactory::getUser($profile->user_id);
          $geo->username = $user->username;

          $this->_db->insertObject($geo->getTableName(), $geo, 'user_id');
        }
        else
        {
          $this->_db->updateObject($geo->getTableName(), $geo, 'user_id');
        }
      break;

      case 26: // ReCaptcha
        $settings = new LovefactorySettings();

        if (!$settings->enable_recaptcha || empty($settings->recaptcha_public_key) || empty($settings->recaptcha_private_key))
        {
          return true;
        }

        require_once(JPATH_COMPONENT_SITE.DS.'lib'.DS.'recaptcha'.DS.'recaptchalib.php');

        $resp = recaptcha_check_answer(
          $settings->recaptcha_private_key,
          $_SERVER['REMOTE_ADDR'],
          $_POST['recaptcha_challenge_field'],
          $_POST['recaptcha_response_field']);

        if (!$resp->is_valid)
        {
          JError::raiseWarning('ERROR', JText::_('FIELD_RECAPTCHA_ERROR_INVALID'));
          return false;
        }
      break;
	  }

	  return true;
	}

	/**
	 * Updates the specific field properties from a POST request
	 *
	 * @return unknown
	 */
	function updateFromRequest()
	{
	  switch ($this->type_id)
	  {
	    case 1: // Text field
	      $maxlength      =& JRequest::getVar('maxlength',          255, 'POST', 'integer');
	      $validation     =& JRequest::getVar('validation_type',    0,   'POST', 'integer');
	      $message        =& JRequest::getVar('validation_message', '',  'POST', 'string');
	      $regexp         =& JRequest::getVar('regexp',             '',  'POST', 'string');
	      $location_field =& JRequest::getVar('location_field',     0,   'POST', 'integer');

	      $maxlength = 255 < $maxlength ? 255 : $maxlength;

	      $this->params = $this->convertToParams(array(
	        'maxlength'          => $maxlength,
	        'validation'         => $validation,
	        'validation_message' => $message,
	        'regexp'             => $regexp,
	        'location_field'     => $location_field,
	      ));
	    break;

	    case 2: // Text area
	      $rows         =& JRequest::getVar('rows',         10, 'POST', 'integer');
	      $cols         =& JRequest::getVar('cols',         40, 'POST', 'integer');
	      $allowed_tags =& JRequest::getVar('allowed_tags', '', 'POST', 'string');

	      $this->params = $this->convertToParams(array(
	        'rows'         => $rows,
	        'cols'         => $cols,
	        'allowed_tags' => $allowed_tags,
	      ));
	    break;

	    case 3: // Drop down (single select)
  	    // Options
	      $options =& JRequest::getVar('options', '', 'POST', 'string');
	      $options = explode("\n", $options);

	      foreach ($options as $i => $option)
	      {
	        $options[$i] = trim(str_replace("\n", '', $option));
	      }

	      $options = implode('*|*', $options);

	      $this->params = $this->convertToParams(array('values'  => $options));
	    break;

	    case 4: // Drop down (multiple select)
	    case 5: // Checkboxes
  	    // Options
	      $options =& JRequest::getVar('options', '', 'POST', 'string');
	      $options = explode("\n", $options);

	      foreach ($options as $i => $option)
	      {
	        $options[$i] = trim(str_replace("\n", '', $option));
	      }

	      $options = implode('*|*', $options);

	      // Display
	      $display =& JRequest::getVar('display', 0, 'POST', 'integer');

	      if (!in_array($display, array(0, 1), true))
	      {
	        $display = 0;
	      }

	      // Query operand
	      $query_operand =& JRequest::getVar('query_operand', 0, 'POST', 'integer');

	      $this->params = $this->convertToParams(array(
	        'values'        => $options,
	        'display'       => $display,
	        'query_operand' => $query_operand)
	      );
	    break;

	    case 6: // Radio buttons
  	    // Options
	      $options =& JRequest::getVar('options', '', 'POST', 'string');
	      $options = explode("\n", $options);

	      foreach ($options as $i => $option)
	      {
	        $options[$i] = trim(str_replace("\n", '', $option));
	      }

	      $options = implode('*|*', $options);

	      // Display
	      $display =& JRequest::getVar('display', 0, 'POST', 'integer');

	      if (!in_array($display, array(0, 1), true))
	      {
	        $display = 0;
	      }

	      $this->params = $this->convertToParams(array(
	        'values'  => $options,
	        'display' => $display)
	      );
	    break;

	    case 7: // Date of birth
	      $minage =& JRequest::getVar('minage', 18, 'POST', 'integer');
	      $maxage =& JRequest::getVar('maxage', 90, 'POST', 'integer');

	      if ($minage < 0)
	      {
	        $minage = 18;
	      }

	      if ($maxage < 0)
	      {
	        $maxage = 90;
	      }

	      if ($minage > $maxage)
	      {
	        $minage = $maxage;
	      }

	      $this->params = $this->convertToParams(array('minage'  => $minage, 'maxage' => $maxage));
	    break;

	    case 9: // Main photo
	      // Link
	      $link =& JRequest::getVar('link', 1, 'POST', 'integer');

	      if (!in_array($link, array(0, 1, 2, 3)))
	      {
	        $link = 1;
	      }

	      // Gallery info
	      $gallery =& JRequest::getVar('gallery', 1, 'POST', 'integer');

	      if (!in_array($gallery, array(0, 1)))
	      {
	        $link = 1;
	      }

	      $this->params = $this->convertToParams(array(
	        'link'    => $link,
	        'gallery' => $gallery,
	      ));
	    break;

	    case 10: // Sex
	    case 11: // Looking for
	      if (10 == $this->type_id)
	      {
	        // Options
	        $options =& JRequest::getVar('options', '', 'POST', 'string');
	        $options = explode("\n", $options);

	        foreach ($options as $i => $option)
	        {
	          $options[$i] = trim(str_replace("\n", '', $option));
	        }

	        $options = implode('*|*', $options);
	      }
	      else
	      {
	        $options = $this->getParam('values');
	        $this->_old_mode = $this->getParam('mode');
	      }

	      // Display
	      $display =& JRequest::getVar('display', 0, 'POST', 'integer');

	      if (!in_array($display, array(0, 1)))
	      {
	        $display = 0;
	      }

	      // Mode
	      $mode =& JRequest::getVar('mode', 0, 'POST', 'integer');

	      $this->params = $this->convertToParams(array(
	        'values'  => $options,
	        'display' => $display,
	        'mode'    => $mode,
	      ));
	    break;

	    case 12: // Username
	      $ajax_check =& JRequest::getVar('ajax_check', 1,  'POST', 'integer');

	      $this->params = $this->convertToParams(array(
	        'ajax_check' => $ajax_check,
	      ));
	    break;

	    case 13: // Online members
	    case 14: // With pictures
	    $label =& JRequest::getVar('label', '', 'POST', 'string');

	    if ('' == $label)
	    {
	      JError::raiseWarning(0, JText::_('FIELD_ERROR_LABEL_EMPTY'));
	      return false;
	    }

	    $this->params = 'values=' . $label;
	    break;

	    case 15: // Last seen
	      $mode =& JRequest::getVar('mode', 0, 'POST', 'integer');

	      if (!in_array($mode, array(0, 1), true))
	      {
	        $mode = 0;
	      }

	      $this->params = $this->convertToParams(array('mode' => $mode));
	    break;

	    case 17: // Online status
	      $online  =& JRequest::getVar('online',  '', 'POST', 'string');
	      $offline =& JRequest::getVar('offline', '', 'POST', 'string');

	      $online  = '' == $online  ? 'online'  : $online;
	      $offline = '' == $offline ? 'offline' : $offline;

	      $this->params = $this->convertToParams(array('online' => $online, 'offline' => $offline));
	    break;

	    case 18: // Email
	      $validation =& JRequest::getVar('validation_type',    0,   'POST', 'integer');
	      $message    =& JRequest::getVar('validation_message', '',  'POST', 'string');
	      $regexp     =& JRequest::getVar('regexp',             '',  'POST', 'string');
	      $ajax_check =& JRequest::getVar('ajax_check',          1,  'POST', 'integer');

	      $this->params = $this->convertToParams(array(
	        'validation'         => $validation,
	        'validation_message' => $message,
	        'regexp'             => $regexp,
	        'ajax_check'         => $ajax_check,
	      ));
	    break;

	    case 20: // Repeat password
	      $message =& JRequest::getVar('validation_message', '',  'POST', 'string');
	      $this->params = $this->convertToParams(array(
	        'validation_message' => $message,
	      ));
	    break;

	    case 23: // Google map location
	      $height        =& JRequest::getVar('height',        100, 'POST', 'integer');
	      $width         =& JRequest::getVar('width',         100, 'POST', 'integer');
	      $show_hide_map =& JRequest::getVar('show_hide_map', 1,   'POST', 'integer');
	      $show_full_map =& JRequest::getVar('show_full_map', 1,   'POST', 'integer');

	      $this->params = $this->convertToParams(array(
	        'height'        => $height,
	        'width'         => $width,
	        'show_hide_map' => $show_hide_map,
	        'show_full_map' => $show_full_map,
	      ));
	    break;

	    case 24: // Distance
	      $search_method =& JRequest::getVar('search_method', 0, 'POST', 'integer');

	      $this->params = $this->convertToParams(array(
	        'search_method' => $search_method,
	      ));
	    break;

	    case 25: // Relationship
	      $show_photo =& JRequest::getVar('show_photo', 0, 'POST', 'integer');

	      $this->params = $this->convertToParams(array(
	        'show_photo' => $show_photo,
	        'values'     => empty($this->title_search) ? $this->title : $this->title_search,
	      ));
	    break;

	    case 26: // ReCaptcha
	      $theme =& JRequest::getVar('theme', 0, 'POST', 'integer');

	      $this->params = $this->convertToParams(array(
	        'theme' => $theme,
	      ));
	    break;

	    case 27: // Spacer
	      $height =& JRequest::getVar('height', 20,  'POST', 'integer');

	      $this->params = $this->convertToParams(array(
	        'height' => $height,
	      ));
	    break;

	    case 28: // Terms and Conditions
	      $terms =& JRequest::getVar('terms', '',  'POST', 'string', 4);
	      $terms = str_replace('=', '&#61;', base64_encode($terms));

	      $this->params = $this->convertToParams(array(
	        'values' => 'TERMS_AND_CONDITIONS_LABEL',
	        'terms'  => $terms,
	      ));
	    break;
	  }

	  return true;
	}

	/**
	 * Updates the common properties from a POST request
	 *
	 * @return unknown
	 */
	function updateCommonFromRequest()
	{
	  $this->title        =& JRequest::getVar('title',        '', 'POST', 'string');
	  $this->title_view   =& JRequest::getVar('title_view',   '', 'POST', 'string');
	  $this->title_edit   =& JRequest::getVar('title_edit',   '', 'POST', 'string');
	  $this->title_search =& JRequest::getVar('title_search', '', 'POST', 'string');
	  $this->show_title   =& JRequest::getVar('show_title',   1,  'POST', 'integer');
	  $this->required     =& JRequest::getVar('required',     0,  'POST', 'integer');
	  $this->description  =& JRequest::getVar('description',  '', 'POST', 'string');
	  $this->published    =& JRequest::getVar('published',    1,  'POST', 'integer');
	  $this->visibility   =& JRequest::getVar('visibility',   1,  'POST', 'integer');

	  if ('' == $this->title)
	  {
	    JError::raiseWarning(0, JText::_('FIELD_ERROR_TITLE_EMPTY'));
	    return false;
	  }

	  $isNew = count($this->_errors) ? true : false;

	  if ((!$isNew && !$this->system) || $isNew)
	  {
	    $types   =  $this->getTypes();
	    $type_id =& JRequest::getVar('type_id', 0, 'POST', 'integer');

	    if (!isset($types[$type_id]))
	    {
	      JError::raiseWarning(0, JText::_('FIELD_ERROR_TYPE_UNDEFINED'));
	      return false;
	    }

	    if (1 == $types[$type_id])
	    {
	      JError::raiseWarning(0, JText::_('FIELD_ERROR_SYSTEM_FIELD_ADD'));
	      return false;
	    }

	    $this->type_id         =  $type_id;
	    $this->user_visibility =& JRequest::getVar('user_visibility', 0, 'POST', 'integer');
	  }

	  return true;
	}

	/**
	 * Retrieves all the field types (system and user)
	 *
	 * @return unknown
	 */
	function getTypes()
	{
	  return $this->getUserTypes() + $this->getSystemTypes();
	}

	/**
	 * Retrieves the user field types
	 *
	 * @return unknown
	 */
	function getUserTypes()
	{
	  $types = array(
	    1  => 0, // Text field
      2  => 0, // Text area
      3  => 0, // Drop down (single select)
      4  => 0, // Drop down (multiple select)
      5  => 0, // Checkboxes
      6  => 0, // Radio buttons
	  );

	  return $types;
	}

	/**
	 * Retrieves the system field types
	 *
	 * @return unknown
	 */
	function getSystemTypes()
	{
	  $types = array(
      7  => 1, // Birth date
      8  => 1, // Membership
      9  => 1, // Main photo
      10 => 1, // Sex
      11 => 1, // Looking for
      12 => 1, // Username
      13 => 1, // Online members (search condition)
      14 => 1, // With pictures (search condition)
      15 => 1, // Last seen
      16 => 1, // No. of photos
      17 => 1, // Online status
      18 => 1, // Email
      19 => 1, // Password
      20 => 1, // Repeat password
      21 => 1, // Main photo upload
      22 => 1, // No. of friends
      23 => 1, // Google map
      24 => 1, // Distance
      25 => 1, // Relationship
      26 => 1, // ReCaptcha
      27 => 1, // Spacer
      28 => 1, // Terms and Conditions
      29 => 1, // Profile rating
	  );

	  return $types;
	}

	/**
	 * Converts specific field parameters to
	 * format to store in database
	 *
	 * @param unknown_type $values
	 * @return unknown
	 */
	function convertToParams($values)
	{
	  $output = '';

	  foreach ($values as $param => $value)
	  {
	    $output .= $param . '=' . $value . "\n";
	  }

	  return $output;
	}

	/**
	 * Gets the default parameter values
	 *
	 * @return unknown
	 */
	function getDefaultValues()
	{
	  global $Itemid;
    $params = explode("\n", $this->params);

    foreach ($params as $param)
    {
      $param = explode('=', trim($param));

      switch ($this->type_id)
      {
        case 28: // Terms and Conditions
          $val = isset($param[1]) ? $param[1] : 'TERMS_AND_CONDITIONS_LABEL';
          return array(JText::sprintf($val, '<a href="' . JRoute::_('index.php?option=com_lovefactory&view=terms&Itemid=' . $Itemid, true)) . '" target="_blank">' . JText::_('TERMS_AND_CONDITIONS') . '</a>');
        break;
      }

      if (count($param) == 2 && $param[0] == 'values')
      {
        return explode('*|*', $param[1]);
      }
    }

    return false;
	}

	/**
	 * Gets a specific parameter value
	 *
	 * @param unknown_type $profile
	 * @return unknown
	 */
	function getValue($profile)
	{
	  if (is_null($profile) && isset($this->_value))
	  {
	    return $this->_value;
	  }

	  $defaults = $this->getDefaultValues();

	  switch ($this->type_id)
	  {
	    case 3:  // Drop down (single select)
	    case 4:  // Drop down (multiple select)
      case 10: // Sex
      case 11: // Looking for

        switch ($this->type_id)
        {
          case 10:
	          $value = isset($profile->sex) ? $profile->sex : '';
	        break;

	        case 11:
	          $value = isset($profile->looking) ? $profile->looking : '';
	        break;

	        default:
	          $value = isset($profile->{'field_' . $this->id}) ? $profile->{'field_' . $this->id} : '';
	        break;
        }

        $multiple = false;
        if (4 == $this->type_id ||
           (11 == $this->type_id && in_array($this->getParam('mode', 1), array(3,4))))
        {
          $multiple = true;
        }

        if (!empty($value))
        {
          $flip = array_flip($defaults);

          if (!$multiple)
          {
            if (11 == $this->type_id)
            {
              $value = str_replace('*|*', '', $value);
            }

            $value = @$flip[$value];
          }
          else
          {
            $value = explode('*|*', $value);
            $value = array_intersect($defaults, $value);
            $value = array_flip($value);
          }
        }
      break;

      case 5: // Checkboxes
      case 6: // Radio buttons
      case 13: // Online users
      case 14: // With pictures
      case 25: // Relationship
        $value = isset($profile->{'field_' . $this->id}) ? $profile->{'field_' . $this->id} : '';

        $value = explode('*|*', $value);
        $value = array_intersect($defaults, $value);
        $value = array_flip($value);
      break;

      case 7: // Date of birth
        $values = isset($profile->{'field_' . $this->id}) ? $profile->{'field_' . $this->id} : '';

        if (is_array($values))
        {
          $value = $values;
        }
        else
        {
          $value = array(
            'year'  => substr($values, 0, 4),
            'month' => substr($values, 4, 2),
            'day'   => substr($values, 6, 2),
          );
        }
      break;

	    default:
	     $value = isset($profile->{'field_' . $this->id}) ? $profile->{'field_' . $this->id} : '';
	    break;
	  }

	  return $value;
	}

	/**
	 * Retreives the description tag formatted for output
	 *
	 * @return unknown
	 */
	function getDescriptionTag()
	{
	  $output = '';

	  if ($this->description)
	  {
	    $output .= '<span class="hasTip" title="' . JText::_($this->title).' :: ' . JText::_($this->description) . '">'
              .  '<img src="' . JURI::root() . 'components/com_lovefactory/assets/images/information.png" class="lovefactory_icon" alt="Information" />'
              .  '</span>';
	  }

	  return $output;
	}

	/**
	 * Gets the field params
	 *
	 * @return unknown
	 */
  function getParams()
	{
	  $array  = array();
    $params = explode("\n", $this->params);

    foreach ($params as $i => $param)
    {
      $param = explode('=', trim($param));

      if (!empty($param[0]))
      {
        $array[$param[0]] = $param[1];
      }
    }

    return $array;
	}

	/**
	 * Gets a specific param
	 *
	 * @param unknown_type $param
	 * @param unknown_type $default
	 * @return unknown
	 */
	function getParam($param, $default = false)
	{
	  $params = $this->getParams();

	  switch ($this->type_id)
	  {
	    case 28: // Terms and Condtions
	      if ('terms' == $param)
	      {
	        $params[$param] = base64_decode(str_replace('&#61;', '=', @$params[$param]));
	      }
	    break;
	  }

	  return isset($params[$param]) ? $params[$param] : $default;
	}

  /**
   * Renders the field for edit
   *
   * @param unknown_type $profile
   * @return unknown
   */
	function renderForEdit($profile = null)
	{
	  $output = $this->_renderEditable('edit', $profile);

	  // User visibility
	  if ($this->user_visibility && !is_null($profile))
	  {
	    //$this->set_user_visibility = isset($this->set_user_visibility) ? $this->set_user_visibility : $this->visibility;
	    $this->set_user_visibility = $profile->{'field_' . $this->id . '_visibility'};

	    $visibilities = array(0 => JText::_('FIELD_HIDDEN'), 1 => JText::_('FIELD_FOR_ALL'), 2 => JText::_('FIELD_ONLY_FOR_FRIENDS'));
	    $output .= '<br /><small> <a href="#" class="lovefactory-field-visibility" rel="' . $this->id . '"><span id="txt_field_visibility_' . $this->id . '">' . $visibilities[$this->set_user_visibility] . '</span></a></small>'
	             . '<input type="hidden" value="' . $this->set_user_visibility . '" name="field_visibility_' . $this->id . '" id="field_visibility_' . $this->id . '" />';
	  }

	  return $output;
	}

	/**
	 * Renders the field for search
	 *
	 * @param unknown_type $profile
	 * @return unknown
	 */
	function renderForSearch($profile = null)
	{
	  return $this->_renderEditable('search', $profile);
	}

	/**
	 * Renders the field for view
	 *
	 * @param unknown_type $profile
	 * @return unknown
	 */
	function _renderViewable($profile = null)
	{
	  switch ($this->type_id)
	  {
	    case 1:  // Text input
	    case 3:  // Drop down (single select)
	      $output = $profile->{'field_' . $this->id};
	      break;

	    case 2:  // Text area
	      $output = nl2br($profile->{'field_' . $this->id});
	      break;

	    case 4: // Drop down (multiple select)
	    case 5: // Checkboxes
	    case 6: // Radio buttons
	      $values = explode('*|*', $profile->{'field_' . $this->id});

	      foreach ($values as $i => $value)
	      {
	        if (empty($value))
	        {
	          unset($values[$i]);
	        }
	        else
	        {
	          $values[$i] = JText::_($value);
	        }
	      }

	      $display = $this->getParam('display') ? ', ' : '<br />';
	      $output  = implode($display, $values);
	      break;

	    case 7: // Date of birth
	      $value = $profile->{'field_' . $this->id};

	      if ('' == $value)
	      {
	        return '-';
	      }

	      $year  = intval(substr($value, 0, 4));
        $month = intval(substr($value, 4, 2));
        $day   = intval(substr($value, 6, 2));

        $age_years  = date('Y') - $year;
        $age_months = date('m') - $month;
        $age_days   = date('d') - $day;

        if ($age_days < 0)
        {
          $age_months = $age_months - 1;
        }

        if ($age_months < 0)
        {
          $age_years = $age_years - 1;
          $age_months = $age_months + 12;
        }

        $output = JText::sprintf('FIELD_YEARS_OLD', $age_years);
	    break;

	    case 8: // Membership
	      $output = $profile->title;
	    break;

	    case 9: // Photos
	      global $Itemid;

	      $settings = new LovefactorySettings();

	      $table =& JTable::getInstance('profile', 'Table');
	      $table->bind($profile);
	      $table->_email = $profile->_email;

	      switch ($this->getParam('link'))
	      {
	        case 0:
	          $link = '';
	        break;

	        case 1:
	          $link = 'index.php?option=com_lovefactory&view=profile&Itemid=' . $Itemid . '&id=' . $table->user_id;
	          $link = '<a href="' . JRoute::_($this->_getBackendLinkForFrontend($link)) . '"></a>';
	        break;

	        case 2:
	          $link = 'index.php?option=com_lovefactory&view=gallery&Itemid=' . $Itemid . '&user_id=' . $table->user_id;
	          $link = '<a href="' . JRoute::_($this->_getBackendLinkForFrontend($link)) . '"></a>';
	        break;

	        case 3:
	          $view =& JRequest::getVar('view', '', 'REQUEST', 'cmd');

	          if ('profile' == $view)
	          {
	            $link = 'index.php?option=com_lovefactory&view=gallery&Itemid=' . $Itemid . '&user_id=' . $table->user_id;
	          }
	          else
	          {
	            $link = 'index.php?option=com_lovefactory&view=profile&Itemid=' . $Itemid . '&id=' . $table->user_id;
	          }

	          $link = '<a href="' . JRoute::_($this->_getBackendLinkForFrontend($link)) . '"></a>';
	        break;
	      }

	      /* @var $table TableProfile */
	      $output = '<div style="background-image: url(\'' . $table->getMainPhotoSource(true) . '\'); background-repeat: no-repeat; height: ' . $settings->thumbnail_max_height . 'px; width: ' . $settings->thumbnail_max_width . 'px;" class="lovefactory-thumbnail">' . $link . '</div>';

	      if ($this->getParam('gallery'))
	      {
          $link = $this->_getBackendLinkForFrontend(JRoute::_('index.php?option=com_lovefactory&view=gallery&Itemid=' . $Itemid . '&user_id=' . $table->user_id));
          if ($profile->photos > 1)
	        {
	          $output .= '<div class="lovefactory-more-photos">' . JText::_('and') . ' <a href="' . $link . '">' . ($profile->photos - 1) . ' ' . JText::_('more') . '</a> ' . (($profile->photos == 2) ? JText::_('photo') : JText::_('photos')) . '</div>';
	        }
	        else
	        {
	          $output .= '<div class="lovefactory-more-photos">' . ' <a href="' . $link . '">' . JText::_('photos gallery') . '</a></div>';
	        }
	      }
	    break;

	    case 10: // Sex
	      $output = $profile->sex;
	    break;

	    case 11: // Looking for
	      $output = $profile->looking;

	      switch ($this->getParam('mode', 1))
	      {
	        case 1:
	        case 2:
	          $output = str_replace('*|*', '', $profile->looking);
	        break;

	        case 3:
	        case 4:
	          $values = explode('*|*', $profile->looking);

	          foreach ($values as $i => $value)
	          {
	            if (empty($value))
	            {
	              unset($values[$i]);
	            }
	            else
	            {
	              $values[$i] = JText::_($value);
	            }
	          }

	          $display = $this->getParam('display') ? ', ' : '<br />';
	          $output  = implode($display, $values);
	        break;
	      }
	    break;

	    case 12: // Username
	      $output = $profile->username;
	    break;

	    case 15: // Last seen
	      require_once(JPATH_COMPONENT_SITE.DS.'lib'.DS.'vendor'.DS.'TheFactoryHelper.class.php');
	      $date =& JFactory::getDate();

	      if ($this->_db->_nullDate == $profile->lastvisitDate)
	      {
	        $output = JText::_('FIELD_LAST_SEEN_NEVER');
	        break;
	      }

	      if (0 == $this->getParam('mode'))
	      {
          $output = TheFactoryHelper::format_date($profile->last_seen_on, 'ago');
	      }
	      else
	      {
	        $output = date('Y-m-d H:i:s', $profile->last_seen_on);
	      }
	    break;

	    case 16: // No. of photos
	      $output = $profile->photos;

	      if (!$this->show_title)
	      {
	        $output .= ' <span style="text-transform: lowercase;">'
	                .  JText::_(1 == $profile->photos ? 'photo' : 'photos')
	                .  '</span>';
	      }

	      $output = '<a href="' . JRoute::_('index.php?option=com_lovefactory&view=gallery&user_id=' . $profile->user_id) . '">' . $output . '</a>';
	    break;

	    case 17: // Online status
	      if ($profile->loggedin)
	      {
	        $output = '<span class="lovefactory-button lovefactory-bullet-green">' . JText::_($this->getParam('online')) . '</span>';
	      }
	      else
	      {
	        $output = '<span class="lovefactory-button lovefactory-bullet-red">' . JText::_($this->getParam('offline')) . '</span>';
	      }
	    break;

	    case 22: // No. of friends
	      $output = $profile->friends;

	      if (!$this->show_title)
	      {
	        $output .= ' <span style="text-transform: lowercase;">'
	                .  JText::_(1 == $profile->friends ? 'friend' : 'friends')
	                .  '</span>';
	      }

	      $output = '<a href="' . JRoute::_('index.php?option=com_lovefactory&view=friends&id=' . $profile->user_id) . '">' . $output . '</a>';
	    break;

	    case 23: // Google map location
	      $settings = new LovefactorySettings();

	      // Check if Google maps are enabled
	      if (!$settings->enable_gmaps)
	      {
	        $output = JText::_('FIELD_GOOGLE_MAPS_NOT_ENABLED');
	        break;
	      }

	      // Check if location is set
        if (0 == $profile->lat && 0 == $profile->lng)
	      {
	        $output = JText::_('FIELD_GOOGLE_MAPS_USER_NOT_DEFINED_LOCATIONS');
	        break;
	      }

        JHTML::script('?file=api&amp;v=2&amp;sensor=false&amp;key=' . $settings->gmaps_api_key, 'http://maps.google.com/maps');
        JHTML::script('profile_gmaps.js', 'components/com_lovefactory/assets/js/views/');

        $maps_display =& JRequest::getVar('lovefactory_show_maps', 1, 'COOKIE', 'integer');

        global $Itemid;
        $document =& JFactory::getDocument();
        $document->addScriptDeclaration('var lf_gmap_location  = { lat: ' . $profile->lat . ', lng: ' . $profile->lng . ' };');
        $document->addScriptDeclaration('var lf_pin_src        = "' . JURI::root() . 'components/com_lovefactory/assets/images/pin.png";');
        $document->addScriptDeclaration('var lf_pin_shadow_src = "' . JURI::root() . 'components/com_lovefactory/assets/images/pin_shadow.png";');
        $document->addScriptDeclaration('var txt_show = "' . JText::_('show') . '";');
        $document->addScriptDeclaration('var txt_hide = "' . JText::_('hide') . '";');

	      $output = '<div id="google-map" style="width: ' . $this->getParam('width', 500) . 'px; height: ' . $this->getParam('height', 300) . 'px; display: ' . ($maps_display ? 'block' : 'none') . ';"></div>'
	              . '<div class="lovefactory_gmaps_extra">'
	              . ($this->getParam('show_hide_map', 1) ? '<a href="#" id="lovefactory-toggle-maps" class="lovefactory-button ' . ($maps_display ? 'lovefactory-toggle-minus' : 'lovefactory-toggle-plus') . '"><span>' . ($maps_display ? JText::_('hide') : JText::_('show')) . '</span> map</a>' : '')
	              . ($this->getParam('show_full_map', 1) && $settings->enable_members_map ? '<a id="lovefactory-full-map" href="' . JRoute::_('index.php?option=com_lovefactory&view=membersmap&Itemid=' . $Itemid) . '" class="lovefactory-button lovefactory-map-go">' . JText::_('Full members map') . '</a>' : '')

	              . '</div>';
	    break;

	    case 24: // Distance
	      $settings = new LovefactorySettings();

	      // Check if Google maps are enabled
	      if (!$settings->enable_gmaps)
	      {
	        $output = JText::_('FIELD_GOOGLE_MAPS_NOT_ENABLED');
	        break;
	      }

        if (JText::_('NOT_AVAILABLE') == $profile->distance)
        {
          $output = $profile->distance;
          break;
        }

	      $output = $profile->distance . ' ' . (!$settings->distances_unit ? JText::_('KM') : JText::_('MI'));
	    break;

	    case 25: // Relationship
	      if (is_null($profile->relationship_id))
	      {
	        $output = JText::_('NOT_AVAILABLE');
	        break;
	      }

	      global $Itemid;
	      $link = '<a href="' . JRoute::_('index.php?option=com_lovefactory&view=profile&id=' . $profile->relationship_id . '&Itemid=' . $Itemid) . '">' . $profile->relationship_username . '</a>';

	      $output = $link;

	      if ($this->getParam('show_photo', 0))
	      {
	        $settings =  new LovefactorySettings();
	        $table    =& JTable::getInstance('profile', 'Table');

	        $table->user_id    = $profile->relationship_id;
	        $table->main_photo = $profile->relationship_main_photo;

	        /* @var $table TableProfile */
	        $photo = '<div style="background-image: url(\'' . $table->getMainPhotoSource(true) . '\'); height: ' . $settings->thumbnail_max_height . 'px; width: ' . $settings->thumbnail_max_width . 'px;" class="lovefactory-thumbnail"></div>';

	        $output = '<div style="text-align: center;">' . $photo . $link . '</div>';
	      }
	    break;

	    case 27: // Spacer
        $output = '<div style="height: '.$this->getParam('height', 20).'px;"></div>';
      break;

	    case 29: // Profile Rating
	      $output = '<span class="lovefactory-button lovefactory-stars"><b>'.$profile->rating.'</b></span>';
	    break;

	    default:
	      $output = $this->type_id;
	    break;
	  }

	  return $output == '' ? '-' : $output;
	}

	function _renderViewableOnlyFriends()
	{
	  switch ($this->type_id)
	  {
	    case 9: // Main photo
	      $settings = new LovefactorySettings();
	      $output = '<div style="background-image: url(components/com_lovefactory/assets/images/love.png); height: ' . $settings->thumbnail_max_height . 'px; width: ' . $settings->thumbnail_max_width . 'px;" class="lovefactory-thumbnail"></div>';
	    break;

	    default:
	      $output = JText::_('FIELD_ONLY_FOR_FRIENDS');
	    break;
	  }

	  return $output;
	}

	function _renderViewableHidden()
	{
	  switch ($this->type_id)
	  {
	    case 9: // Main photo
	      $settings = new LovefactorySettings();
	      $output = '<div style="background-image: url(components/com_lovefactory/assets/images/love.png); height: ' . $settings->thumbnail_max_height . 'px; width: ' . $settings->thumbnail_max_width . 'px;" class="lovefactory-thumbnail"></div>';
	    break;

	    default:
	      $output = JText::_('FIELD_HIDDEN');
	    break;
	  }

	  return $output;
	}

	function renderForView($profile = null, $display_hidden)
	{
	  // If it's my profile, show the field regardless of other conditions
	  $user =& JFactory::getUser();
	  if (null != $profile && $user->id == $profile->user_id)
	  {
	    return $this->_renderViewable($profile);
	  }

	  if (1 == $display_hidden)
	  {
	    switch ($this->_visibility)
	    {
	      case 0: // Hidden
	        return $this->_renderViewableHidden();
	      break;

	      case 2: // For friends
	        if (!$profile->is_friend)
	        {
	          return $this->_renderViewableOnlyFriends();
	        }
	      break;
	    }
	  }

	  return $this->_renderViewable($profile);
	}

	/**
	 * Helper to render editable fields (edit and search)
	 *
	 * @param unknown_type $mode
	 * @param unknown_type $profile
	 * @return unknown
	 */
	function _renderEditable($mode, $profile = null, $id = null)
	{
	  $value    = $this->getValue($profile);
	  $defaults = $this->getDefaultValues();

	  switch ($this->type_id)
	  {
	    case 1:  // Text input
	      switch ($this->getParam('validation'))
	      {
	        case 0:
	          $rel = 0;
	        break;

	        case 1:
	          $message = '' != JText::_($this->getParam('validation_message')) ? JText::_($this->getParam('validation_message')) : JText::_('this field must be numeric');
	          $rel = htmlentities(1 . '___' . $message);
	        break;

	        case 2:
	          $message = '' != JText::_($this->getParam('validation_message')) ? JText::_($this->getParam('validation_message')) : JText::_('this field failed regexp validation');
	          $rel = htmlentities(2 . '___' . $message . '___' . $this->getParam('regexp'));
	        break;
	      }

	      $output = '<input type="text" name="field_' . $this->id . '" id="field_' . $this->id . '" value="' . $value . '" alt="' . $rel . '" maxlength="' . $this->getParam('maxlength', 255) . '" />';
	    break;

	    case 2: // Text area
	      $output = '<textarea name="field_' . $this->id . '" id="field_' . $this->id . '" rows="' . $this->getParam('rows', 10) . '" cols="' . $this->getParam('cols', 40) . '">' . $value . '</textarea>';

	      if ('edit' == $mode && '' != $this->getParam('allowed_tags'))
	      {
	        $output .= '<div class="lovefactory-info">' . JText::_('FIELD_ALLOWED_HTML_TAGS') . ': ' . $this->getParam('allowed_tags') . '</div>';
	      }
	    break;

	    case 3:  // Drop down (single select)
	    case 4:  // Drop down (multiple select)
        $multiple = $this->type_id == 4;
        $options  = array();

        if (!$multiple && !$this->getParam('no_blank'))
        {
          $options[] = JHTML::_('select.option', -1, '&nbsp;');
        }

        foreach ($defaults as $key => $val)
        {
          if ('' != $val)
          {
            $options[] = JHTML::_('select.option', $key, JText::_($val));
          }
        }

        $id       = 'field_' . $this->id;
        $id       = $multiple ? $id . '[]' : $id;
        $multiple = $multiple ? 'multiple' : '';

        $output = JHTML::_('select.genericlist', $options, $id, $multiple, 'value', 'text', $value);
      break;

      case 10: // Sex
        $settings = new LovefactorySettings();
        $id = is_null($id) ? 'field_' . $this->id : $id;

        if ('search' != $mode && !$settings->gender_change && null != @$profile->sex)
        {
          $output = $profile->sex;
          break;
        }

        switch ($this->getParam('mode'))
        {
          case 0: // Drop down (single select)
            $options[] = JHTML::_('select.option', -1, '&nbsp;');
            foreach ($defaults as $key => $val)
            {
              $options[] = JHTML::_('select.option', $key, JText::_($val));
            }

            $output = JHTML::_('select.genericlist', $options, $id, '', 'value', 'text', $value);
          break;

          case 1: // Radio buttons
            $array = array();
            $separator = $this->getParam('display') ? '' : '<br />';

            foreach ($this->getDefaultValues() as $key => $val)
            {
              $checked = $key === $value ? 'checked="checked"' : '';

              $array[] = '<input ' . $checked . ' type="radio" name="' . $id . '" value="' . $key . '" id="' . $id . '_' . $key . '" />'
                       . '<label for="' . $id . '_' . $key . '">' . JText::_(html_entity_decode($val)) . '</label>';
            }

            $output = implode($separator, $array);
          break;
        }
      break;

      case 11: // Looking
        $id = is_null($id) ? 'field_' . $this->id : $id;

        switch ($this->getParam('mode'))
        {
          default:
          case 1: // Drop down (single select)
            $options[] = JHTML::_('select.option', -1, '&nbsp;');
            foreach ($defaults as $key => $val)
            {
              $options[] = JHTML::_('select.option', $key, JText::_($val));
            }

            $output = JHTML::_('select.genericlist', $options, $id, '', 'value', 'text', $value);
          break;

          case 2: // Radio buttons
            $array = array();
            $separator = $this->getParam('display') ? '' : '<br />';

            foreach ($this->getDefaultValues() as $key => $val)
            {
              $checked = $key === $value ? 'checked="checked"' : '';

              $array[] = '<input ' . $checked . ' type="radio" name="' . $id . '" value="' . $key . '" id="' . $id . '_' . $key . '" />'
                       . '<label for="' . $id . '_' . $key . '">' . JText::_(html_entity_decode($val)) . '</label>';
            }

            $output = implode($separator, $array);
          break;

          case 3: // Drop down (multiple select)
            $multiple = true;
            $options  = array();

            foreach ($defaults as $key => $val)
            {
              $options[] = JHTML::_('select.option', $key, JText::_($val));
            }

            $id       = $multiple ? $id . '[]' : $id;
            $multiple = $multiple ? 'multiple' : '';

            $output = JHTML::_('select.genericlist', $options, $id, $multiple, 'value', 'text', $value);
          break;

          case 4: // Checkboxes
            $display = $this->getParam('display') ? '' : '<br />';
            $array   = array();

            foreach ($defaults as $key => $val)
            {
              $checked = @in_array($key, $value) ? 'checked="checked"' : '';

              $array[] = '<input ' . $checked . ' type="checkbox" name="' . $id . '[' . $key . ']" id="' . $id . '_' . $key . '" />'
                       . '<label for="' . $id . '_' . $key . '">' . ucfirst(JText::_(html_entity_decode($val))) . '</label>';
            }

            $output = implode($display, $array);
          break;
        }
      break;

      case 5:  // Checkboxes
      case 13: // Online
      case 14: // With pictures
      case 28: // Terms and conditions
        $display = $this->getParam('display') ? '' : '<br />';
        $array   = array();

        foreach ($defaults as $key => $val)
        {
          $checked = @in_array($key, $value) ? 'checked' : '';

          $array[] = '<input ' . $checked . ' type="checkbox" name="field_' . $this->id . '[' . $key . ']" id="field_' . $this->id . '_' . $key . '" />'
                   . '<label for="field_' . $this->id . '_' . $key . '">' . ucfirst(JText::_(html_entity_decode($val))) . '</label>';
        }

        $output = implode($display, $array);
      break;

      case 6: // Radio buttons
        $display = $this->getParam('display') ? '' : '<br />';
        $array   = array();

        foreach ($this->getDefaultValues() as $key => $val)
        {
          $checked = in_array($key, $value) ? 'checked' : '';

          $array[] = '<input ' . $checked . ' type="radio" name="field_' . $this->id . '" value="' . $key . '" id="field_' . $this->id . '_' . $key . '" />'
                   . '<label for="field_' . $this->id . '_' . $key . '">' . JText::_(html_entity_decode($val)) . '</label>';
        }

        $output = implode($display, $array);
      break;

      case 7: // Date birth
        $output = '';
        $params = $this->getParams();

        $min_year = date('Y') - $params['maxage'];
        $max_year = date('Y') - $params['minage'];

        switch ($mode)
        {
          case 'edit':

            // Day
            $options = array();
            for ($i = 1; $i < 32; $i++)
            {
              $options[] = JHTML::_('select.option', $i);
            }
            $output .= JHTML::_('select.genericlist', $options, 'field_' . $this->id . '[day]', '', 'value', 'text', $value['day']);

            $output .= '&nbsp;/&nbsp;';

            // Month
            $options = array();
            for ($i = 1; $i < 13; $i++)
            {
              $options[] = JHTML::_('select.option', $i);
            }
            $output .= JHTML::_('select.genericlist', $options, 'field_' . $this->id . '[month]', '', 'value', 'text', $value['month']);

            $output .= '&nbsp;/&nbsp;';

            // Year
            $options = array();
            for ($i = $max_year; $i >= $min_year; $i--)
            {
              $options[] = JHTML::_('select.option', $i);
            }
            $output .= JHTML::_('select.genericlist', $options, 'field_' . $this->id . '[year]', '', 'value', 'text', $value['year']);
          break;

          case 'search':
            // Starting year
            $options = array();
            for ($i = date('Y') - $max_year; $i <= date('Y') - $min_year; $i++)
            {
              $options[] = JHTML::_('select.option', $i);
            }
            $output .= JHTML::_('select.genericlist', $options, 'field_' . $this->id . '[starting]', '', 'value', 'text', isset($value['starting']) ? $value['starting'] : date('Y') - $max_year);

            $output .= '&nbsp;' . JText::_('and') . '&nbsp;';

            // Ending year
            $options = array();
            for ($i = date('Y') - $max_year; $i <= date('Y') - $min_year; $i++)
            {
              $options[] = JHTML::_('select.option', $i);
            }
            $output .= JHTML::_('select.genericlist', $options, 'field_' . $this->id . '[ending]', '', 'value', 'text', isset($value['ending']) ? $value['ending'] : date('Y') - $min_year);
          break;
        }

      break;

      case 12: // Username
        if ('edit' == $mode)
        {
          $ajax_check = 1 == $this->getParam('ajax_check', 1) ? 'ajax-check' : '';
	        $output = '<input type="text" class="' . $ajax_check . '" name="field_' . $this->id . '" id="field_' . $this->id . '" value="' . $value . '" rel="0" />';
	        if ($this->getParam('ajax_check', 1))
          {
            $output .= '&nbsp;<span class="loader" style="display: none;"><img src="components/com_lovefactory/assets/images/ajax-loader.gif" /></span><span class="response"></span>';
          }
        }
        else
        {
          $output = '<input type="text" name="field_' . $this->id . '" id="field_' . $this->id . '" value="' . $value . '" rel="0" />';
        }
	    break;

      case 18: // Email
        switch ($this->getParam('validation'))
	      {
	        case 2:
	          $message = '' != JText::_($this->getParam('validation_message')) ? JText::_($this->getParam('validation_message')) : JText::_('this field failed regexp validation');
	          $rel = htmlentities(2 . '___' . $message . '___' . $this->getParam('regexp'));
	        break;

	        default:
	          $rel = 0;
	        break;
	      }

	      $ajax_check = 1 == $this->getParam('ajax_check', 1) ? 'ajax-check' : '';
        $output = '<input type="text" class="' . $ajax_check . ' email" name="field_' . $this->id . '" id="field_' . $this->id . '" value="' . $value . '" alt="' . $rel . '" />';
        if ($this->getParam('ajax_check', 1))
        {
          $output .= '&nbsp;<span class="loader" style="display: none;"><img src="components/com_lovefactory/assets/images/ajax-loader.gif" /></span><span class="response"></span>';
        }
      break;

      case 19: // Password
        $output = '<input type="password" name="field_' . $this->id . '" id="field_' . $this->id . '" value="" />';
      break;

      case 20: // Repeat password
        $output = '<input type="password" name="field_' . $this->id . '" id="field_' . $this->id . '" value="" rel="' . htmlentities($this->getParam('validation_message', JText::_('Wrong password'))) . '" />';
      break;

      case 21: // Main photo upload
        $settings = new LovefactorySettings();

        $output = '<div class="lovefactory-thumbnail" id="lovefactory-main-photo-upload" style="width: ' . $settings->thumbnail_max_width . 'px; height: ' . $settings->thumbnail_max_height . 'px; vertical-align: middle;"></div>'
                . '<div id="swfupload-control"><input type="button" id="button" /></div>'
                . '<div id="progressbar" style="display: none;"></div>'
                . '<input type="hidden" name="main_photo_upload" id="main_photo_upload" value="0" />';
      break;

      case 23: // Google map location
        $settings = new LovefactorySettings();

        if (!$settings->enable_gmaps)
        {
          $output = JText::_('FIELD_GOOGLE_MAPS_NOT_ENABLED');
          break;
        }

        JHTML::script('?file=api&amp;v=2&amp;sensor=false&amp;key=' . $settings->gmaps_api_key, 'http://maps.google.com/maps');
        JHTML::script('edit_gmaps.js', 'components/com_lovefactory/assets/js/views/');

        $profile->lat = !isset($profile->lat) ? 0 : $profile->lat;
        $profile->lng = !isset($profile->lng) ? 0 : $profile->lng;

        $location_set = 0 != $profile->lat && 0 != $profile->lng;

        $document =& JFactory::getDocument();
        $document->addScriptDeclaration('var lf_gmap_location  = { lat: ' . $profile->lat . ', lng: ' . $profile->lng . ' };');
        $document->addScriptDeclaration('var lf_pin_src        = "' . JURI::root() . 'components/com_lovefactory/assets/images/pin.png";');
        $document->addScriptDeclaration('var lf_pin_shadow_src = "' . JURI::root() . 'components/com_lovefactory/assets/images/pin_shadow.png";');

        $output = '<div id="google-map" style="width: ' . $this->getParam('width', 500) . 'px; height: ' . $this->getParam('height', 300) . 'px"></div>'
                . '<div class="lovefactory_gmaps_extra">'
	              . '  <a style="display: ' . ($location_set ? 'inline' : 'none') . '" id="lovefactory-remove-map-location" href="#" class="lovefactory-button lovefactory-map-delete">' . JText::_('FIELD_REMOVE_MY_LOCATION') . '</a>'
	              . '</div>'
                . '<input type="hidden" id="gmap_lat" name="gmap_lat" value="' . $profile->lat . '" />'
                . '<input type="hidden" id="gmap_lng" name="gmap_lng" value="' . $profile->lng . '" />';
      break;

      case 24: // Distance
        $settings = new LovefactorySettings();

        // Check if Google maps are enabled
        if (!$settings->enable_gmaps)
        {
          $output = JText::_('FIELD_GOOGLE_MAPS_NOT_ENABLED');
          break;
        }

        // Check if user is logged in
        $user =& JFactory::getUser();
        if ($user->guest)
        {
          $output = JText::_('FIELD_GOOGLE_MAPS_LOGIN');
          break;
        }

        // Check if user has set own location
        $geo =& JTable::getInstance('geo', 'Table');
        $geo->load($user->id);

        if (0 == $geo->lat && 0 == $geo->lng)
        {
          $output = JText::_('FIELD_GOOGLE_MAPS_SET_LOCATION');
          break;
        }

        $settings = new LovefactorySettings();

        $output = '<input type="text" name="field_' . $this->id . '" id="field_' . $this->id . '" value="' . $value . '" style="width: 50px;" /> '
                . (!$settings->distances_unit ? JText::_('KM') : JText::_('MI'));
      break;

      case 25: // Relationship
        $defaults = array_keys($defaults);
        $defaults = $defaults[0];

        $checked = @in_array($defaults, $value) ? 'checked' : '';

        $title = !empty($this->title_search) ? $this->title_search : $this->title;
        $output = '<input ' . $checked . ' type="checkbox" name="field_' . $this->id . '[0]" id="field_' . $this->id . '" />'
                . '<label for="field_' . $this->id . '">' . ucfirst(JText::_(html_entity_decode($title))) . '</label>';
      break;

      case 26: // ReCaptcha
        $settings = new LovefactorySettings();

        if (!$settings->enable_recaptcha)
        {
          $output = JText::_('FIELD_RECAPTCHA_NOT_ENABLED');
        }
        elseif (empty($settings->recaptcha_public_key) || empty($settings->recaptcha_private_key))
        {
          $output = JText::_('FIELD_RECAPTCHA_NOT_CONFIGURED');
        }
        else
        {
          require_once(JPATH_COMPONENT_SITE.DS.'lib'.DS.'recaptcha'.DS.'recaptchalib.php');

          if ($this->getParam('theme', 0))
          {
            $themes = array('custom', 'red', 'white', 'blackglass', 'clean');
            $theme = '<script type="text/javascript">var RecaptchaOptions = { theme : \''.$themes[$this->getParam('theme', 0)].'\' };</script>';

            $output = $theme . recaptcha_get_html($settings->recaptcha_public_key);
          }
          else
          {
            require_once(JPATH_COMPONENT_SITE.DS.'lib'.DS.'html'.DS.'fields.php');

            $output = JHTML::_('fields.recaptcha', $settings->recaptcha_public_key);
          }
        }
      break;

      case 27: // Spacer
        $output = '<div style="height: '.$this->getParam('height', 20).'px;"></div>';
      break;

	    default:
	      $output = $this->type_id;
	    break;
	  }

	  if (!$this->show_title && $this->required)
	  {
	    $output .= '<span class="lovefactory-required-field">*</span>';
	  }

	  return $output;
	}

	/**
	 * Renders the field label for edit
	 *
	 * @return unknown
	 */
	function renderLabelForEdit()
	{
	  $title = !empty($this->title_edit) ? $this->title_edit : $this->title;

	  if (empty($title))
	  {
	    return false;
	  }

	  $description = $this->getDescriptionTag();

	  $for = 'for="field_' . $this->id . '"';

	  switch ($this->type_id)
	  {
	    case 7: // Date of birth
	    case 5: // Checkboxes
	    case 6: // Radio buttons
	    case 23: // Google maps location
	      $for = '';
	    break;

	    case 11: // Looking for
	      $for = in_array($this->getParam('mode', 1), array(2,4)) ? '' : $for;
	    break;

	    case 26: // ReCaptcha
	      $for = 'for="recaptcha_response_field"';
	    break;
	  }

	  $output = '<label ' . $for . '>' . ucfirst(JText::_($title)) . ':';

	  if ($this->required)
	  {
	    $output .= '<span class="lovefactory-required-field">*</span>';
	  }

	  $output .= '</label>';

	  $output .= $description;

	  return $output;
	}

	/**
	 * Renders the field label for search
	 *
	 * @return unknown
	 */
	function renderLabelForSearch()
	{
	  $title = !empty($this->title_search) ? $this->title_search : $this->title;

	  if (empty($title))
	  {
	    return false;
	  }

	  $for = 'for="field_' . $this->id . '"';

	  switch ($this->type_id)
	  {
	    case 7: // Date of birth
	    case 5: // Checkboxes
	    case 6: // Radio buttons
	      $for = '';
	    break;

	    case 11: // Looking for
	      $for = in_array($this->getParam('mode', 1), array(2,4)) ? '' : $for;
	    break;
	  }

	  $output = '<label ' . $for . '>' . ucfirst(JText::_($title)) . ':</label>';

	  return $output;
	}

	/**
	 * Renders the field label for view
	 *
	 * @param unknown_type $profile
	 * @return unknown
	 */
	function renderLabelForView($profile = null)
	{
	  $title = !empty($this->title_view) ? $this->title_view : $this->title;

	  if (empty($title))
	  {
	    return false;
	  }

	  $output = '<label>' . ucfirst(JText::_($title)) . ':</label>';

	  return $output;
	}

	/**
	 * Renders the field required message
	 *
	 * @return unknown
	 */
	function renderRequiredMessage()
	{
	  if (!$this->required && 0 == $this->getParam('validation'))
	  {
	    return false;
	  }

	  $output = '<div id="error_field_' . $this->id . '" style="display: none;" class="lovefactory-error-field">'
	          . '  <span class="lovefactory-button lovefactory-bullet-error"></span>'
	          . '</div>';

	  return $output;
	}

	function showField($profile = null, $display_hidden)
	{
	  switch ($this->type_id)
	  {
	    case 27: // Spacer
	      return true;
	    break;

	    case 29: // Profile Rating
	      $settings = new LovefactorySettings();
	      if (!$settings->enable_rating)
	      {
	        return false;
	      }
	    break;
	  }

	  $this->_visibility = $this->user_visibility ? @$profile->{'field_' . $this->id . '_visibility'} : $this->visibility;

	  // If it's my profile, show the field regardless of other conditions
	  $user =& JFactory::getUser();
	  if (null != $profile && isset($profile->user_id) && $user->id == $profile->user_id)
	  {
	    return true;
	  }

	  if (1 == $display_hidden)
	  {
	    return true;
	  }

	  switch ($this->_visibility)
	  {
	    case 0: // Hidden
	      return false;
	    break;

	    case 2: // For friends
	      if (!$profile->is_friend)
	      {
	        return false;
	      }
	    break;
	  }

	  return true;
	}

	function showTitle($mode)
	{
	  if (!$this->show_title)
	  {
	    return false;
	  }

	  switch ($mode)
	  {
	    case 'search':
	      if (in_array($this->type_id, array(25)))
	      {
	        return false;
	      }

	      return true;
	    break;

	    default:
	      return true;
	    break;
	  }
	}

	/**
	 * Gets the query for search
	 *
	 * @param unknown_type $post
	 * @return unknown
	 */
	function getSearchConditionQuery($post = null, $type = 'where')
	{
	  if (!isset($post['field_' . $this->id]))
	  {
	    return false;
	  }

	  if ('where' == $type && 13 == $this->type_id)
	  {
	    return false;
	  }
	  else if ('having' == $type && 13 != $this->type_id)
	  {
	    return false;
	  }

	  $condition = $post['field_' . $this->id];
	  $defaults  = $this->getDefaultValues();
	  $output    = false;

	  switch ($this->type_id)
	  {
	    case 1: // Text input
	    case 2: // Text area
	      if ('' != $condition)
	      {
	        $condition = $this->_db->getEscaped($condition);

	        if ($this->user_visibility)
	        {
	          // User set visibility
	          $output = ' CASE p.field_' . $this->id . '_visibility'
	                  . '   WHEN 1 THEN p.field_' . $this->id. ' LIKE "%' . $condition . '%"'
                    . '   WHEN 2 THEN (p.field_' . $this->id. ' LIKE "%' . $condition . '%" AND IF (f.id IS NOT NULL AND f.pending = 0, 1, 0) = 1)'
                    . '   WHEN 0 THEN 1 = 0'
                    . ' END';
	        }
	        elseif (1 == $this->visibility)
	        {
	          // Visible for all
	          $output = ' p.field_' . $this->id .' LIKE "%' . $condition . '%"';
	        }
	        elseif (2 == $this->visibility)
	        {
	          // Visible for friends
	          $output = ' ((p.field_' . $this->id .' LIKE "%' . $condition . '%") AND IF (f.id IS NOT NULL AND f.pending = 0, 1, 0) = 1)';
	        }
	      }
	    break;

	    case 3: // Drop down (single select)
	      if (!isset($defaults[$condition]))
	      {
	        break;
	      }

	      $condition = $defaults[$condition];

  	    if ($this->user_visibility)
	      {
	        // User set visibility
	        $output = ' CASE p.field_' . $this->id . '_visibility'
	                . '   WHEN 1 THEN p.field_' . $this->id. ' LIKE "%' . $condition . '%"'
	                . '   WHEN 2 THEN (p.field_' . $this->id. ' LIKE "%' . $condition . '%" AND IF (f.id IS NOT NULL AND f.pending = 0, 1, 0) = 1)'
	                . '   WHEN 0 THEN 1 = 0'
	                . ' END';
	      }
	      elseif (1 == $this->visibility)
	      {
	        // Visible for all
	        $output = ' p.field_' . $this->id .' LIKE "%' . $condition . '%"';
	      }
	      else if (2 == $this->visibility)
	      {
	        // Visible for friends
	        $output = ' ((p.field_' . $this->id .' LIKE "%' . $condition . '%") AND IF (f.id IS NOT NULL AND f.pending = 0, 1, 0) = 1)';
	      }
	    break;

	    case 4: // Drop down (multiple select)
	    case 5: // Checkboxes
	      $array = array();

	      if (5 == $this->type_id)
	      {
	        $condition = array_keys($condition);
	      }

	      foreach ($condition as $value)
	      {
	        $value = '*|*'.$defaults[$value].'*|*';

	        if ($this->user_visibility)
	        {
	          // User set visibility
	          $array[] = ' (CASE p.field_' . $this->id . '_visibility'
	                   . '   WHEN 1 THEN p.field_' . $this->id. ' LIKE "%' . $value. '%"'
	                   . '   WHEN 2 THEN (p.field_' . $this->id. ' LIKE "%' . $value . '%" AND IF (f.id IS NOT NULL AND f.pending = 0, 1, 0) = 1)'
	                   . '   WHEN 0 THEN 1 = 0'
	                   . ' END)';
	        }
	        elseif (1 == $this->visibility)
	        {
	          // Visible for all
	          $array[] = ' p.field_' . $this->id .' LIKE "%' . $value . '%"';
	        }
	        else if (2 == $this->visibility)
	        {
	          // Visible for friends
	          $array[] = ' ((p.field_' . $this->id .' LIKE "%' . $value . '%") AND IF (f.id IS NOT NULL AND f.pending = 0, 1, 0) = 1)';
	        }
	      }

	      if (count($array))
	      {
	        $operand = $this->getParam('query_operand', 0) ? ' AND ' : ' OR ';
	        $output = '(' . implode($operand, $array) . ')';
	      }
	    break;

	    // Radio buttons
	    case 6:
	      $condition = $defaults[$condition];
	      if ($this->user_visibility)
	      {
	        // User set visibility
	        $output = ' CASE p.field_' . $this->id . '_visibility'
	                . '   WHEN 1 THEN p.field_' . $this->id. ' LIKE "%' . $condition . '%"'
	                . '   WHEN 2 THEN (p.field_' . $this->id. ' LIKE "%' . $condition . '%" AND IF (f.id IS NOT NULL AND f.pending = 0, 1, 0) = 1)'
	                . '   WHEN 0 THEN 1 = 0'
	                . ' END';
	      }
	      elseif (1 == $this->visibility)
	      {
	        // Visible for all
	        $output = ' p.field_' . $this->id .' LIKE "%' . $condition . '%"';
	      }
	      else if (2 == $this->visibility)
	      {
	        // Visible for friends
	        $output = ' ((p.field_' . $this->id .' LIKE "%' . $condition . '%") AND IF (f.id IS NOT NULL AND f.pending = 0, 1, 0) = 1)';
	      }
	    break;

	    // Date of birth
	    case 7:
	      $starting = (int)$condition['starting'];
	      $ending   = (int)$condition['ending'];

	      $starting = (date('Y') - $starting)   . date('md');
        $ending   = (date('Y') - $ending - 1) . date('md');

        $output = ' (p.field_' . $this->id . ' <= ' . $starting
                . ' AND '
                . ' p.field_' . $this->id . ' >= ' . $ending . ')';
	    break;

	    // Sex
	    case 10:
	      if (isset($defaults[$condition]))
	      {
	        //$output = ' p.looking = "' . $defaults[$condition] . '"';
	        $output = ' p.looking LIKE "%*|*' . $defaults[$condition] . '*|*%"';
	      }
	    break;

	    // Looking for
	    case 11:
	      if (is_array($condition))
	      {
	        if (4 == $this->getParam('mode'))
	        {
	          $condition = array_keys($condition);
	        }

	        $array = array();
	        foreach ($condition as $i => $option)
	        {
	          $array[] = ' p.sex = "' . $defaults[$option] . '"';
	        }

	        $output = '(' . implode(' OR ', $array) . ')';
	      }
	      else
	      {
	        if (isset($defaults[$condition]))
	        {
	          $output = ' p.sex = "' . $defaults[$condition] . '"';
	        }
	      }
	    break;

	    // Username
	    case 12:
	      if ('' != $condition)
	      {
	        $condition = $this->_db->getEscaped($condition);

	        $output = ' u.username LIKE "%' . $condition . '%"';
	      }
	    break;

	    // Online users
	    case 13:
	      $output = 'having' == $type ? ' COUNT(se.userid) <> 0' : false;
	    break;

	    // With pictures
	    case 14:
	      $output = ' p.photos > 0';
	    break;

	    case 24: // Distance
	      $user =& JFactory::getUser();

	      // Check if user is guest
	      if ($user->guest)
	      {
	        return false;
	      }

	      $geo =& JTable::getInstance('geo', 'Table');
	      $geo->load($user->id);

	      // Check if user has defined own location
	      if (0 == $geo->lat && 0 == $geo->lng)
	      {
	        return false;
	      }

	      $settings  = new LovefactorySettings();
	      $unit      = !$settings->distances_unit ? 1.6 : 1;
	      $condition = $this->_db->getEscaped($condition);

	      if ('' == $condition)
	      {
	        return false;
	      }

	      $condition = intval($condition);

	      if (!$this->getParam('search_method', 0))
	      {
	        $latitude = $geo->lat;
	        $longitude = $geo->lng;
	        $radius = $condition;
	        $constant = 69.172 * $unit;

	        $delta_latitude = ($radius / $constant);
	        $lat1 = $latitude + $delta_latitude;
	        $lat2 = $latitude - $delta_latitude;
	        $delta_longitude = ($radius / (cos($latitude) * $constant));
	        $long1 = $longitude + $delta_longitude;
	        $long2 = $longitude - $delta_longitude;

	        $output = ' IF (g.lat <> 0 AND g.lng <> 0, (g.lat BETWEEN '.$lat2.' AND '.$lat1.') AND (g.lng BETWEEN '.$long2.' AND '.$long1.'), 1 = 0)';
	      }
	      else
	      {
	        $output = ' IF (lat <> 0 AND lng <> 0, ROUND((((acos(sin((' . $geo->lat.'*pi()/180)) * sin((lat*pi()/180)) + cos(('.$geo->lat.'*pi()/180)) * cos((lat*pi()/180)) * cos((('.$geo->lng.' - lng)*pi()/180))))*180/pi())*60*1.1515) * ' . $unit . ') <= ' . $condition . ', 1 = 0)';
	      }
	    break;

	    case 25: // Relationship
	      $output = ' p.relationship <> 0';
	    break;
	  }

	  return $output;
	}

	/**
	 * Stores the field
	 *
	 * @param unknown_type $updateNulls
	 */
	function store($updateNulls = false)
	{
	  parent::store($updateNulls);

	  // If field is sex, then update the looking for field
    if (10 == $this->type_id)
    {
      $params = array('values' => $this->getParam('values'));

      $query = ' SELECT f.id'
             . ' FROM #__lovefactory_fields f'
             . ' WHERE f.type_id = 11';
      $this->_db->setQuery($query);
      $id = $this->_db->loadResult();

      $field =& JTable::getInstance('field', 'Table');
      $field->load($id);

      foreach ($field->getParams() as $param => $value)
      {
        if ('values' != $param)
        {
          $params[$param] = $value;
        }
      }

      $field->params = $field->convertToParams($params);
      $field->store();
    }

	  if (!$this->system || 7 == $this->type_id)
    {
      // Update database table
      $field_types = array(
        1 => 'VARCHAR(255)',
        2 => 'MEDIUMTEXT',
        3 => 'MEDIUMTEXT',
        4 => 'MEDIUMTEXT',
        5 => 'MEDIUMTEXT',
        6 => 'MEDIUMTEXT',
        7 => 'VARCHAR(8)',
      );

      $query = ' SHOW COLUMNS FROM #__lovefactory_profiles';
      $this->_db->setQuery($query);
      $columns = $this->_db->loadObjectList('Field');

      // Check if field exists
      if (array_key_exists('field_' . $this->id, $columns))
      {
        // Field exists, but does it have the right type?
        if ($columns['field_' . $this->id]->Type != strtolower($field_types[$this->type_id]))
        {
          $query = ' ALTER TABLE #__lovefactory_profiles'
                 . ' CHANGE field_' . $this->id . ' field_' . $this->id . ' ' . $field_types[$this->type_id] . ' NOT NULL ';
          $this->_db->setQuery($query);
          $this->_db->query();
        }
      }
      else
      {
        // Field doesn't exist, add it
        $query = ' ALTER TABLE #__lovefactory_profiles'
               . ' ADD field_' . $this->id . ' ' . $field_types[$this->type_id] . ' NOT NULL,'
               . ' ADD field_' . $this->id . '_visibility TINYINT(1) DEFAULT \'1\'';
        $this->_db->setQuery($query);
        $this->_db->query();
      }
    }

    // If field is looking for and we are changing from multiple select to single select
    if (11 == $this->type_id)
    {
      if (in_array($this->getParam('mode'), array(1,2)) &&
          in_array($this->_old_mode, array(3,4)))
      {
        $query = ' SELECT p.looking, p.user_id'
               . ' FROM #__lovefactory_profiles p';
        $this->_db->setQuery($query);
        $profiles = $this->_db->loadObjectList();

        foreach ($profiles as $profile)
        {
          $exploded = explode('*|*', $profile->looking);
          if (3 <= count($exploded) || 1 == count($exploded))
          {
            $i = 1 == count($exploded) ? 0 : 1;

            $looking = '*|*' . $exploded[$i] . '*|*';
            $query = ' UPDATE #__lovefactory_profiles'
                   . ' SET looking = "' . $looking . '"'
                   . ' WHERE user_id = ' . $profile->user_id;
            $this->_db->setQuery($query);
            $this->_db->query();
          }
        }
      }
    }
	}

	/**
	 * Deletes the field
	 *
	 * @param unknown_type $oid
	 */
	function delete($oid = null)
	{
	  // Delete the column from the profile table
	  $query = ' ALTER TABLE #__lovefactory_profiles'
	         . ' DROP field_' . $this->id . ', '
	         . ' DROP field_' . $this->id . '_visibility';
	  $this->_db->setQuery($query);
	  $this->_db->query();

	  // Delete the field from the pages
	  $query = ' SELECT *'
	         . ' FROM #__lovefactory_pages';
	  $this->_db->setQuery($query);
	  $pages = $this->_db->loadObjectList();

	  foreach ($pages as $page)
	  {
	    if (preg_match('/(^|#)(\d+)_(\d+)_' . $this->id . '(#|$)/', $page->fields, $matches))
	    {
	      $replace_with = ($matches[1] == '#' && $matches[4] == '#') ? '#' : '';

	      $query = ' UPDATE #__lovefactory_pages'
	             . ' SET fields = "' . str_replace($matches[0], $replace_with, $page->fields) . '"'
	             . ' WHERE id = ' . $page->id;
	      $this->_db->setQuery($query);
	      $this->_db->query();
	    }
	  }

	  return parent::delete($oid);
	}

	function getMainPhoto()
	{
	  $query = ' SELECT f.*'
	         . ' FROM #__lovefactory_fields f'
	         . ' WHERE f.type_id = 9';
	  $this->_db->setQuery($query);

	  $this->bind($this->_db->loadObject());
	}

	function _getBackendLinkForFrontend($link)
	{
	  global $mainframe;

	  if ('site' != $mainframe->getName())
	  {
	    $link = JURI::root() . $link;
	  }

	  return $link;
	}
}
