<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FrontendModelInteraction extends JModel
{
  function send()
  {
    $user    =& JFactory::getUser();
    $id      =& JRequest::getVar('user_id', 0, 'REQUEST', 'integer');
    $type_id =& JRequest::getVar('type_id', 0, 'REQUEST', 'integer');

    require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'settings.php');
    $settings = new LovefactorySettings();

    // Check if interactions are enabled
    if (!$settings->enable_interactions)
    {
      $this->setError(JText::_('INTERACTIONS_ERROR_INTERACTIONS_NOT_ENABLED'));
      return false;
    }

    // Check if current interaction is enabled
    $interaction_enabled = false;

    switch ($type_id)
    {
      // Kiss
      case 1:
        if ($settings->enable_interaction_kiss)
        {
          $interaction_enabled = true;
        }
      break;

      // Wink
      case 3:
        if ($settings->enable_interaction_wink)
        {
          $interaction_enabled = true;
        }
      break;

      // Hug
      case 5:
        if ($settings->enable_interaction_hug)
        {
          $interaction_enabled = true;
        }
      break;
    }

    if (!$interaction_enabled)
    {
      $this->setError(JText::_('INTERACTIONS_ERROR_INTERACTION_NOT_ENABLED'));
      return false;
    }

    // Check if there are any interactions left for the day
    $query = ' SELECT p.*, m.max_interactions_per_day'
           . ' FROM #__lovefactory_profiles p'
           . ' LEFT JOIN #__lovefactory_memberships_sold m ON m.id = p.membership_sold_id'
           . ' WHERE p.user_id = ' . $user->id;
    $this->_db->setQuery($query);
    $sender = $this->_db->loadObject();

    $statistics_per_day =& $this->getTable('statisticsPerDay');
    $max_interactions   =  $sender->max_interactions_per_day;
    $already_sent       =  $statistics_per_day->getInteractions($user->id);

    if ($already_sent >= $max_interactions && $max_interactions != -1)
    {
      $this->setError(JText::_('INTERACTIONS_ERROR_NO_INTERACTIONS_LEFT'));
      $this->_code = 2;

      return false;
    }

    $profile =& $this->getTable('profile');
    $profile->load($id);

    if ($profile->_is_new)
    {
      $this->setError(JText::_('INTERACTIONS_ERROR_USER_NOT_FOUND'));
      return false;
    }

    if ($id == $user->id)
    {
      $this->setError(JText::_('INTERACTIONS_ERROR_CANNOT_SEND_TO_SELF'));
      return false;
    }

    // Check if the user is blacklisted
    $blacklist =& JModel::getInstance('blacklist', 'FrontendModel');
    $blacklisted = $blacklist->isBlacklisted($user->id, $profile->user_id);

    if ($blacklisted)
    {
      $this->setError(JText::_('INTERACTIONS_ERROR_BLACKLISTED'));
      return false;
    }

    $date =& JFactory::getDate();
    $interaction =& $this->getTable('interaction');

    $interaction->receiver_id = $id;
    $interaction->sender_id   = $user->id;
    $interaction->date        = $date->toMySql();
    $interaction->type_id     = $type_id;

    $interaction->store();

    $statistics_per_day->updateInteractions($user->id);

    $mailer =& JModel::getInstance('mailer', 'FrontendModel');
    $mailer->sendNotification($profile->user_id, 'interaction');

    return true;
  }

  function respond()
  {
    require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'settings.php');
    $settings = new LovefactorySettings();

    // Check if interactions are enabled
    if (!$settings->enable_interactions)
    {
      JError::raiseWarning('ERROR', JText::_('INTERACTIONS_ERROR_INTERACTIONS_NOT_ENABLED'));
      return false;
    }

    $user =& JFactory::getUser();
    $id   =& JRequest::getVar('id', 0, 'GET', 'integer');

    if (($user->guest))
    {
      JError::raiseWarning('ERROR', JText::_('INTERACTIONS_ERROR_PLEASE_LOGIN'));
      return false;
    }

    $interaction =& $this->getTable('interaction');
    $interaction->load($id);

    if ($interaction->receiver_id != $user->id)
    {
      JError::raiseWarning('ERROR', JText::_('INTERACTIONS_ERROR_INTERACTION_NOT_FOUND'));
      return false;
    }

    // Check if current interaction is enabled
    $interaction_enabled = false;

    switch ($interaction->type_id)
    {
      // Kiss
      case 1:
        if ($settings->enable_interaction_kiss)
        {
          $interaction_enabled = true;
        }
        break;

      // Wink
      case 3:
        if ($settings->enable_interaction_wink)
        {
          $interaction_enabled = true;
        }
        break;

      // Hug
      case 5:
        if ($settings->enable_interaction_hug)
        {
          $interaction_enabled = true;
        }
        break;
    }

    if (!$interaction_enabled)
    {
      JError::raiseWarning('ERROR', JText::_('INTERACTIONS_ERROR_INTERACTION_NOT_ENABLED'));
      return false;
    }

    // Check if there are any interactions left for the day
    $query = ' SELECT p.*, m.max_interactions_per_day'
           . ' FROM #__lovefactory_profiles p'
           . ' LEFT JOIN #__lovefactory_memberships_sold m ON m.id = p.membership_sold_id'
           . ' WHERE p.user_id = ' . $user->id;
    $this->_db->setQuery($query);
    $sender = $this->_db->loadObject();

    $statistics_per_day =& $this->getTable('statisticsPerDay');
    $max_interactions   =  $sender->max_interactions_per_day;
    $already_sent       =  $statistics_per_day->getInteractions($user->id);

    if ($already_sent >= $max_interactions && $max_interactions != -1)
    {
      $settings = new LovefactorySettings();

      if (!$settings->invalid_membership_action)
      {
        JError::raiseWarning('REDIRECT_NO_MEMBERSHIP', JText::_('INTERACTIONS_ERROR_NO_INTERACTIONS_LEFT'));
      }
      $this->_code = 2;

      return false;
    }

    // Check if the user is blacklisted
    $blacklist   =& JModel::getInstance('blacklist', 'FrontendModel');
    $blacklisted =  $blacklist->isBlacklisted($user->id, $interaction->sender_id);

    if ($blacklisted)
    {
      JError::raiseWarning('ERROR', JText::_('INTERACTIONS_ERROR_BLACKLISTED'));
      return false;
    }

    // Check if user exists
    $receiver =& $this->getTable('profile', 'Table');
    $receiver->load($interaction->sender_id);

    if (!$receiver->validated)
    {
      JError::raiseWarning(0, JText::_('INTERACTIONS_ERROR_USER_NOT_FOUND'));
      return false;
    }

    $date     =& JFactory::getDate();
    $response =& $this->getTable('interaction');

    $response->receiver_id = $interaction->sender_id;
    $response->sender_id   = $interaction->receiver_id;
    $response->date        = $date->toMySql();
    $response->type_id     = $interaction->type_id + 1;

    $response->store();

    $statistics_per_day->updateInteractions($user->id);

    require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'settings.php');
    $settings = new LovefactorySettings();

    $mailer =& JModel::getInstance('mailer', 'FrontendModel');
    $mailer->sendNotification($response->receiver_id, 'interaction');

    return true;
  }
}