<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FrontendModelMailer extends JModel
{
  function __construct()
  {
    parent::__construct();

    $this->settings = new LovefactorySettings();
  }

  function sendNotification($receiver_id, $type, $variables = array())
  {
    if (!$this->settings->enable_user_email_notify)
    {
      return false;
    }

    $types = array(
      'message'           => 'notification_new_message_enabled',
      'rating'            => 'notification_new_rating_enabled',
      'interaction'       => 'notification_new_interaction_enabled',
      'comment'           => 'notification_new_comment_enabled',
      'photo_comment'     => 'notification_new_photo_comment_enabled',
      'change_membership' => 'notification_change_membership_enabled',
    );

    if (!$this->settings->$types[$type])
    {
      return false;
    }

    $user = $this->getUser($receiver_id);

    if (!$user->alerts)
    {
      return false;
    }

    $subject = $this->getSubject($type, $user, $variables);
    $body    = $this->getBody($type, $user, $variables);

    $mail =& JFactory::getMailer();

    $mail->addRecipient($user->email);
    $mail->setSubject($subject);
    $mail->setBody($body);

    if ($this->settings->html_notifications)
    {
      $mail->isHtml(true);
    }

    $mail->send();
  }

  function getUser($receiver_id)
  {
    $query = ' SELECT p.alerts, u.email, u.username, u.id'
           . ' FROM #__lovefactory_profiles p'
           . ' LEFT JOIN #__users u ON u.id = p.user_id'
           . ' WHERE p.user_id = ' . $receiver_id;
    $this->_db->setQuery($query);

    return $this->_db->loadObject();
  }

  function getSubject($type, $user, $variables = array())
  {
    switch ($type)
    {
      case 'message':
        $subject = str_replace('%%username%%', $user->username, $this->settings->notification_new_message_subject);
        break;

      case 'rating':
        $subject = str_replace('%%username%%', $user->username, $this->settings->notification_new_rating_subject);
        break;

      case 'interaction':
        $subject = str_replace('%%username%%', $user->username, $this->settings->notification_new_interaction_subject);
        break;

      case 'comment':
        $subject = str_replace('%%username%%', $user->username, $this->settings->notification_new_comment_subject);
        break;

      case 'photo_comment':
        $search  = array('%%username%%',    '%%link%%');
        $replace = array($user->username, JRoute::_('index.php?option=com_lovefactory&view=photo&id=' . $variables['photo_id'], false, -1));

        $subject = str_replace($search, $replace, $this->settings->notification_new_photo_comment_subject);
        break;

      case 'end_membership':
        $search  = array('%%username%%',  '%%days%%');
        $replace = array($user->username, $variables['days_left']);

        $subject = str_replace($search, $replace, $this->settings->notification_end_membership_subject);
        break;

      case 'change_membership':
        $search  = array('%%receiver%%',  '%%member%%',         '%%membership%%',         '%%until%%');
        $replace = array($user->username, $variables['member'], $variables['membership'], $variables['end_membership']);

        $subject = str_replace($search, $replace, $this->settings->notification_change_membership_subject);
        break;
    }

    return $subject;
  }

  function getBody($type, $user, $variables = array())
  {
    switch ($type)
    {
      case 'message':
        $token = $this->getTokenString($user->id);
        $link = 'index.php?option=com_lovefactory&view=inbox' . $token;

        $search  = array('%%username%%', '%%link%%');
        $replace = array($user->username, JRoute::_($link, false, -1));

        $body = str_replace($search, $replace, $this->settings->notification_new_message_message);
        break;

      case 'rating':
        $token = $this->getTokenString($user->id);
        $link  = 'index.php?option=com_lovefactory&view=profile&id='.$user->id.$token;

        $search  = array('%%username%%', '%%link%%');
        $replace = array($user->username, JRoute::_($link, false, -1));

        $body = str_replace($search, $replace, $this->settings->notification_new_rating_message);
        break;

      case 'interaction':
        $token = $this->getTokenString($user->id);
        $link = 'index.php?option=com_lovefactory&view=interactions' . $token;

        $search  = array('%%username%%', '%%link%%');
        $replace = array($user->username, JRoute::_($link, false, -1));

        $body = str_replace($search, $replace, $this->settings->notification_new_interaction_message);
        break;

      case 'comment':
        $token = $this->getTokenString($user->id);
        $link  = 'index.php?option=com_lovefactory&view=profile&id='.$user->id.$token;

        $search  = array('%%username%%', '%%link%%');
        $replace = array($user->username, JRoute::_($link, false, -1));

        $body = str_replace($search, $replace, $this->settings->notification_new_comment_message);
        break;

      case 'photo_comment':
        $token = $this->getTokenString($user->id);
        $link = JRoute::_('index.php?option=com_lovefactory&view=photo&id=' . $variables['photo_id'] . $token, false, -1);

        $search  = array('%%username%%',  '%%link%%');
        $replace = array($user->username, $link);

        $body = str_replace($search, $replace, $this->settings->notification_new_photo_comment_message);
        break;

      case 'end_membership':
        $search  = array('%%username%%',  '%%days%%');
        $replace = array($user->username, $variables['days_left']);

        $body = str_replace($search, $replace, $this->settings->notification_end_membership_message);
        break;

      case 'change_membership':
        $search  = array('%%receiver%%',  '%%member%%',         '%%membership%%',         '%%until%%');
        $replace = array($user->username, $variables['member'], $variables['membership'], $variables['end_membership']);

        $body = str_replace($search, $replace, $this->settings->notification_change_membership_message);
        break;
    }

    if ($this->settings->html_notifications)
    {
      $body = nl2br($body);
    }

    return $body;
  }

  function sendMultipleNotifications($type, $variables = array())
  {
    $settings = new LovefactorySettings();

    switch ($type)
    {
      case 'change_membership':
        if (!$settings->notification_change_membership_enabled)
        {
          return false;
        }

        $receivers = $this->settings->notification_change_membership_receivers;
        $member    = $this->getUser($variables['member_id']);

        $variables['member'] = $member->username;
      break;
    }

    foreach ($receivers as $receiver_id)
    {
      $user = $this->getUser($receiver_id);

      $subject = $this->getSubject($type, $user, $variables);
      $body    = $this->getBody($type, $user, $variables);

      $mail =& JFactory::getMailer();

      $mail->addRecipient($user->email);
      $mail->setSubject($subject);
      $mail->setBody($body);

      if ($this->settings->html_notifications)
      {
        $mail->isHtml(true);
      }

      $mail->send();
    }
  }

  function getTokenString($user_id)
  {
    $settings   =  new LovefactorySettings();
    $dispatcher =& JDispatcher::getInstance();

    $results = $dispatcher->trigger('FactoryTokenAuthCreateToken', array('parameters' => array('user_id' => $user_id)));

    return $results ? '&' . $results[0] : '';
  }
}