<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FrontendModelMaintenance extends JModel
{
  var $_settings;
  var $_default_membership;

  function __construct()
  {
    parent::__construct();

    $query = ' SELECT m.*'
           . ' FROM #__lovefactory_memberships_sold m'
           . ' WHERE `default` = 1';
    $this->_db->setQuery($query);
    $this->_default_membership = $this->_db->loadObject();
  }

  function doMaintenance()
  {
    $this->checkMemberships();
  }

  // Tasks to be performed
  function checkMemberships()
  {
    $query = ' SELECT DISTINCT(p.user_id), p.membership_sold_id, p.main_photo, p.friends, p.photos'
           . ' FROM #__lovefactory_memberships_sold s'
           . ' LEFT JOIN #__lovefactory_profiles p ON p.membership_sold_id = s.id'
           . ' WHERE UNIX_TIMESTAMP(s.end_membership) < ' . time()
           . ' AND s.id <> ' . $this->_default_membership->id
           . ' AND s.expired = 0'
           . ' AND s.end_membership <> "0000-00-00 00:00:00"';
    $this->_db->setQuery($query);
    $users = $this->_db->loadObjectList();

    if (!count($users))
    {
      return;
    }

    $profile         =& $this->getTable('profile');
    $membership_sold =& $this->getTable('membershipsold');

    foreach ($users as $user)
    {
      // Get the new membership
      $new_membership = $this->getNewMembershipForUser($user);

      // Adjust photos, friends and top friends
      $this->adjustPhotos($user,  $new_membership);
      $this->adjustFriends($user, $new_membership);
      $this->adjustTopFriends($user, $new_membership);
      $this->adjustJoinedGroups($user, $new_membership);

      $profile->loadAndMembership($user->user_id);
      $membership_sold->load($new_membership->id);

      $profile->changeMembershipTo($membership_sold);
    }
  }

  // Helpers
  function getNewMembershipForUser($user)
  {
    $query = ' SELECT m.*'
           . ' FROM #__lovefactory_memberships_sold m'
           . ' WHERE m.user_id = ' . $user->user_id
           . ' AND m.expired = 0'
           . ' AND m.id <> ' . $user->membership_sold_id
           . ' AND m.end_membership < ' . time()
           . ' ORDER BY m.id DESC'
           . ' LIMIT 0,1';
    $this->_db->setQuery($query);
    $new_membership = $this->_db->loadObject();

    return $new_membership ? $new_membership : $this->_default_membership;
  }

  function adjustPhotos($user, $new_membership)
  {
    if ($new_membership->max_photos == -1 || $user->photos <= $new_membership->max_photos)
    {
      return false;
    }

    $query = ' SELECT p.id, p.user_id, p.filename'
           . ' FROM #__lovefactory_photos p'
           . ' WHERE p.user_id = ' . $user->user_id
           . ' ORDER BY RAND()'
           . ' LIMIT 0,' . ($user->photos - $new_membership->max_photos);
    $this->_db->setQuery($query);
    $photos = $this->_db->loadObjectList();

    foreach ($photos as $photo)
    {
      $table =& $this->getTable('photo', 'Table');
      $table->bind($photo);

      /* @var $table TablePhoto */
      $table->delete();
    }

    return true;
  }

  function adjustFriends($user, $new_membership)
  {
    if ($new_membership->max_friends == -1 || $user->friends <= $new_membership->max_friends)
    {
      return false;
    }

    $query = ' SELECT f.id, f.sender_id, f.receiver_id'
           . ' FROM #__lovefactory_friends f'
           . ' WHERE f.sender_id = ' . $user->user_id
           . ' OR f.receiver_id = ' . $user->user_id
           . ' AND f.pending = 0'
           . ' AND f.type = 1'
           . ' ORDER BY RAND()'
           . ' LIMIT 0,' . ($user->friends - $new_membership->max_friends);
    $this->_db->setQuery($query);
    $friends = $this->_db->loadObjectList();

    foreach ($friends as $friend)
    {
      $table =& $this->getTable('friend', 'Table');
      $table->bind($friend);

      /* @var $table TableFriend */
      $table->remove();
    }

    return true;
  }

  function adjustTopFriends($user, $new_membership)
  {
    if ($new_membership->top_friends == -1)
    {
      return false;
    }

    // Get no. of top friends
    $query = ' SELECT COUNT(1)'
           . ' FROM #__lovefactory_friends f'
           . ' WHERE (f.sender_id = ' . $user->user_id . ' OR f.receiver_id = ' . $user->user_id . ')'
           . ' AND f.type = 1';
    $this->_db->setQuery($query);
    $top_friends = $this->_db->loadResult();

    if ($top_friends <= $new_membership->top_friends)
    {
      return false;
    }

    // Get random Top Friends to remove
    $query = ' SELECT f.*'
           . ' FROM #__lovefactory_friends f'
           . ' WHERE (f.sender_id = ' . $user->user_id . ' OR f.receiver_id = ' . $user->user_id . ')'
           . ' AND f.type = 1'
           . ' ORDER BY RAND()'
           . ' LIMIT 0,' . ($top_friends - $new_membership->top_friends);
    $this->_db->setQuery($query);
    $top_friends = $this->_db->loadObjectList();

    foreach ($top_friends as $friend)
    {
      $table =& $this->getTable('friend', 'Table');
      $table->bind($friend);

      /* @var $table TableFriend */
      $table->status = 0;

      $table->store();
    }
  }

  function adjustJoinedGroups($user, $new_membership)
  {
    // Get all the joine groups, that user is not the owner
    $query = ' SELECT COUNT(1)'
           . ' FROM #__lovefactory_group_members m'
           . ' LEFT JOIN #__lovefactory_groups g ON g.id = m.group_id'
           . ' WHERE m.user_id = ' . $user->user_id
           . ' AND g.user_id <> ' . $user->user_id;
    $this->_db->setQuery($query);
    $joined = $this->_db->loadResult();

    if ($joined <= $new_membership->groups_join)
    {
      return false;
    }

    // Get random joined groups to remove
    $query = ' SELECT m.id'
           . ' FROM #__lovefactory_group_members m'
           . ' LEFT JOIN #__lovefactory_groups g ON g.id = m.group_id'
           . ' WHERE m.user_id = ' . $user->user_id
           . ' AND g.user_id <> ' . $user->user_id
           . ' LIMIT 0,' . ($joined - $new_membership->groups_join);
    $this->_db->setQuery($query);
    $joined = $this->_db->loadObjectList();

    $table =& $this->getTable('GroupMember');

    foreach ($joined as $group)
    {
      $table->delete($group->id);
    }

    return true;
  }
}