<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FrontendModelMessage extends JModel
{
  function send($ajax = false)
  {
    $text  =& JRequest::getVar('text',  '', 'POST', 'string');
    $title =& JRequest::getVar('title', '', 'POST', 'string');
    $to    =& JRequest::getVar('to',    '', 'POST', 'string');

    if ('' == $text)
    {
      $this->setError(JText::_('MESSAGE_TASK_SEND_ERROR_REQUIRED_FIELDS'));
      return false;
    }

    if (!$ajax && '' == $to)
    {
      $this->setError(JText::_('MESSAGE_TASK_SEND_ERROR_REQUIRED_FIELDS'));
      return false;
    }

    $profile =& $this->getTable('profile');

    if (!$ajax)
    {
      $profile->loadUsername($to);
    }
    else
    {
      $user_id =& JRequest::getVar('user_id', 0, 'POST', 'integer');
      $profile->load($user_id);
    }

    if ($profile->_is_new)
    {
      $this->setError(JText::_('MESSAGE_TASK_SEND_ERROR_USER_NOT_FOUND'));
      return false;
    }

    $user =& JFactory::getUser();
    if ($user->id == $profile->user_id)
    {
      $this->setError(JText::_('MESSAGE_TASK_SEND_ERROR_SEND_TO_SELF'));
      return false;
    }

    $reply_to =& JRequest::getVar('reply_to', 0, 'POST', 'integer');
    if ($reply_to && $ajax)
    {
      $original =& $this->getTable('message', 'Table');
      $original->load($reply_to);

      $title = JText::sprintf('MESSAGE_REPLY_TITLE', $original->title);
    }

    // Check if the user is blacklisted
    $blacklist   =& JModel::getInstance('blacklist', 'FrontendModel');
    $blacklisted =  $blacklist->isBlacklisted($user->id, $profile->user_id);

    if ($blacklisted)
    {
      $this->setError(JText::_('MESSAGE_TASK_SEND_ERROR_BLACKLISTED'));
      return false;
    }

    $message =& $this->getTable('message');

    // Check the number of messages already sent today
    require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'settings.php');
    $settings = new LovefactorySettings();

    $statistics_per_day =& $this->getTable('statisticsPerDay');

    $query = ' SELECT p.*, m.max_messages_per_day'
           . ' FROM #__lovefactory_profiles p'
           . ' LEFT JOIN #__lovefactory_memberships_sold m ON m.id = p.membership_sold_id'
           . ' WHERE p.user_id = ' . $user->id;
    $this->_db->setQuery($query);
    $sender = $this->_db->loadObject();

    $max_messages = $sender->max_messages_per_day;
    $already_sent = $statistics_per_day->getMessages($user->id);

    if ($already_sent < $max_messages || $max_messages == -1)
    {
      $message->send($profile->user_id, $user->id, $title, $text);
      $statistics_per_day->updateMessages($user->id);

      $mailer =& JModel::getInstance('mailer', 'FrontendModel');
      $mailer->sendNotification($profile->user_id, 'message');
    }
    else
    {
      $this->setError(JText::_('MESSAGE_TASK_SEND_ERROR_UPGRADE'));
      $this->_code = $settings->invalid_membership_action ? 2 : 0;

      return false;
    }

    return true;
  }

  function delete()
  {
    $user =& JFactory::getUser();
    $id   =& JRequest::getVar('id', 0, 'GET', 'integer');

    $message =& $this->getTable('message');
    $message->load($id);

    $this->redirect = ($user->id == $message->sender_id) ? 'outbox' : 'inbox';

    if (($user->id != $message->sender_id) &&
        ($user->id != $message->receiver_id))
    {
      JError::raiseWarning('ERROR', JText::_('MESSAGE_TASK_DELETE_ERROR_NOT_FOUND'));
      return false;
    }

    $message->delete();

    return true;
  }

  function report()
  {
    $id   =& JRequest::getVar('id',     0, 'POST', 'integer');
    $text =& JRequest::getVar('text',  '', 'POST', 'string');
    $user =& JFactory::getUser();

    $message =& $this->getTable('message');
    $message->load($id);

    if ($user->id != $message->receiver_id)
    {
      $this->setError(JText::_('MESSAGE_TASK_REPORT_NOT_FOUND'));
      return false;
    }

    // Set message status as reported
    $message->reported = 1;
    $message->store();

    // Save the report
    $report =& $this->getTable('report');
    $date   =& JFactory::getDate();

    $report->reporting_id = $user->id;
    $report->reported_id  = $id;
    $report->user_id      = $message->sender_id;
    $report->type_id      = 1;
    $report->comment      = $text;
    $report->text         = $message->text;
    $report->date         = $date->toMySql();

    $report->store();

    return true;
  }

  function getMessage()
  {
    $id   =& JRequest::getVar('id', 0, 'GET', 'integer');
    $user =& JFactory::getUser();

    $query = ' SELECT m.*, s.username AS sender_username, r.username AS receiver_username,'
           . '   b.id AS blacklisted, IF (m.sender_id = ' . $user->id . ', 1, 0) AS my_message'
           . ' FROM #__lovefactory_messages m'
           . ' LEFT JOIN #__users s ON s.id = m.sender_id'
           . ' LEFT JOIN #__users r ON r.id = m.receiver_id'
           . ' LEFT JOIN #__lovefactory_blacklist b ON b.sender_id = ' . $user->id . ' AND b.receiver_id = m.sender_id'
           . ' WHERE m.id = ' . $id;
    $this->_db->setQuery($query);
    $message = $this->_db->loadObject();

    if (!$message ||
        ($user->id != $message->receiver_id &&
        $user->id != $message->sender_id))
    {
      return false;
    }

    $table =& $this->getTable('message', 'Table');
    $table->bind($message);

    $table->_sender_username   = $message->sender_username;
    $table->_receiver_username = $message->receiver_username;
    $table->_blacklisted       = $message->blacklisted;
    $table->_my_message        = $message->my_message;

    if ('' == $table->title)
    {
      $table->title = JText::_('MESSAGE_UNTITLED');
    }

    if (!$table->_my_message && $table->unread)
    {
      $table->unread = 0;
      $table->store();
    }

    return $table;
  }

  function getTo()
  {
    $to         =& JRequest::getVar('to',         0, 'GET', 'integer');
    $message_id =& JRequest::getVar('message_id', 0, 'GET', 'integer');

    if (0 != $to)
    {
      $query = ' SELECT u.username'
             . ' FROM #__users u'
             . ' WHERE u.id = ' . $to;
      $this->_db->setQuery($query);
      $message = $this->_db->loadObject();

      return $message ? $message : false;
    }
    elseif (0 != $message_id)
    {
      $user =& JFactory::getUser();

      $query = ' SELECT u.username, CONCAT("' . JText::_('RE:') . '", m.title) AS title'
             . ' FROM #__lovefactory_messages m'
             . ' LEFT JOIN #__users u ON u.id = m.sender_id'
             . ' WHERE m.id = ' . $message_id
             . ' AND m.receiver_id = ' . $user->id;
      $this->_db->setQuery($query);
      $message =& $this->_db->loadObject();

      return $message ? $message : false;
    }

    return false;
  }
}