<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FrontendModelMyGallery extends JModel
{
  // Common
  function delete()
  {
    $media_id   =& JRequest::getVar('media_id', 0, 'POST', 'integer');
    $media_type =& JRequest::getVar('media_type', 'photo', 'POST', 'string');
    $user       =& JFactory::getUser();

    if (!in_array($media_type, array('photo', 'video')))
    {
      $this->setError(JText::_('MY_GALLERY_TASK_DELETE_MEDIA_TYPE_NOT_FOUND'));
      return false;
    }

    $media =& $this->getTable($media_type, 'Table');
    $media->load($media_id);

    if (count($media->_errors) || $media->user_id != $user->id)
    {
      $this->setError(JText::_('MY_GALLERY_TASK_DELETE_NOT_FOUND'));
      return false;
    }

    $media->delete();

    return true;
  }

  function deleteBatch()
  {
    // Get request parameters
    $media_ids  =& JRequest::getVar('media_ids',  array(), 'POST', 'array');
    $media_type =& JRequest::getVar('media_type', '',      'POST', 'string');

    // Validate media type
    if (!in_array($media_type, array('photo', 'video')))
    {
      $this->setError(JText::_('MY_GALLERY_TASK_DELETE_MEDIA_TYPE_NOT_FOUND'));
      return false;
    }

    // Prepare to delete
    JArrayHelper::toInteger($media_ids);
    $this->_deleted = array();

    // Parse and delete each emdia
    foreach ($media_ids as $media_id)
    {
      JRequest::setVar('media_id', $media_id, 'POST');

      if (!$this->delete())
      {
        return false;
      }

      // Add deleted media to list
      $this->_deleted[] = $media_id;
    }

    return true;
  }

  function saveOrder()
  {
    $media_type =& JRequest::getVar('type', 'photo', 'POST', 'string');

    if (!in_array($media_type, array('photo', 'video')))
    {
      return false;
    }

    $serialize =& JRequest::getVar('serialize', '', 'POST', 'string');
    $user      =& JFactory::getUser();
    $array     =  array();
    $exploded  =  explode('&', $serialize);

    foreach ($exploded as $var)
    {
      $var = explode('[]=', $var);

      if (!isset($array[$var[0]]))
      {
        $array[$var[0]] = array();
      }

      $array[$var[0]][] = $var[1];
    }

    $types = array('public' => 0, 'friends' => 1, 'private' => 2);
    foreach ($array as $type => $ids)
    {
      JArrayHelper::toInteger($ids);

      foreach ($ids as $ordering => $id)
      {
        $table =& $this->getTable($media_type, 'Table');
        $table->load($id);

        // Check if photo is valid
        if (count($this->_errors) || $table->user_id != $user->id)
        {
          continue;
        }

        // Media is valid
        $table->status   = $types[$type];
        $table->ordering = $ordering;

        $table->store();
      }
    }

    if ('photo' == $media_type)
    {
      // Update profile photos
      $profile =& $this->getTable('profile', 'Table');
      $profile->load($user->id);
      $profile->photos = isset($array['public']) ? count($array['public']) : 0;
      $profile->store();
    }

    return true;
  }

  // Photo
  function upload()
  {
    // Get uploading user
    $user = $this->getUploadUser();

	  if ($user->guest)
	  {
	    $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_LOGIN'));
	    $this->_error_code = 1;
	    return false;
	  }

    $total   =& $this->getTotalItems('photos', $user);
    $allowed =& $this->getMaximumAllowedItems('photos', $user);

    // Check if user can upload
    if (-1 != $allowed && $total >= $allowed)
    {
      $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_UPGRADE'));
      $this->_error_code = 2;
      return false;
    }

    $photo = JRequest::getVar('photo', array(), 'FILES', 'array');

    if (!count($photo))
    {
      $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_TOO_LARGE'));
      $this->_error_code = 3;
      return false;
    }

    // File size over the limit
    if ($photo['size'] == 0)
    {
      $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_SIZE_EXCEEDED'));
      $this->_error_code = 4;
      return false;
    }

    // Not an image
    $image = getimagesize($photo["tmp_name"]);
    if (false === $image)
    {
      $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_NOT_IMAGE'));
      $this->_error_code = 5;
      return false;
    }

    // Not a valid image
    if (!in_array($image['mime'], array('image/jpeg', 'image/gif', 'image/png')))
    {
      $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_NOT_IMAGE'));
      $this->_error_code = 5;
      return false;
    }

    // All is ok
    jimport('joomla.filesystem.file');

    require_once(JPATH_COMPONENT_SITE.DS.'resizeIMG.php');
    require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'settings.php');

    $types    =  $this->getTypes();
    $settings =  new LovefactorySettings();
    $date     =& JFactory::getDate();
    $type     =& JRequest::getVar('type', 'public', 'POST', 'string');
    $status   =  $types[$type];

    $extension = strtolower(JFile::getExt($photo['name']));
    $filename  = md5($user->id . time() . rand(0, 1000000)) . '.' . $extension;

    $table =& $this->getTable('photo', 'Table');

    /* @var $table TablePhoto */
    $table->user_id    = $user->id;
    $table->filename   = $filename;
    $table->date_added = $date->toMySql();
    $table->status     = $status;
    $table->ordering   = $table->getNextOrder('user_id = ' . $user->id . ' AND status = ' . $status);

    $table->store();

    // Check if user folder exists
    $this->checkUserFolder($user->id);

    $user_folder = $this->getUserFolder($user->id);

    // Photo upload & resize
    $filepath = JPath::clean($user_folder.DS.$filename);
    $resize = new RESIZEIMAGE($photo['tmp_name']);
    $resize->resize_scale($settings->photo_max_height, $settings->photo_max_width, $filepath);
    $resize->close();

    // Thumbnail upload & resize
    $filepath = JPath::clean($user_folder.DS.'thumb_'.$filename);
    $resize = new RESIZEIMAGE($photo['tmp_name']);
    $resize->resize_scale($settings->thumbnail_max_height, $settings->thumbnail_max_width, $filepath);
    $resize->close();

    // JSON response
    $types = array(0 => 'public', 1 => 'friends', 2 => 'private');
    $this->_thumbnail_path = JURI::root().'components/com_lovefactory/storage/photos/'.$user->id.'/thumb_'.$filename;
    $this->_photo_id       = $table->id;
    $this->_type           = $type;

    // Wallpage action
    $action =& $this->getTable('action', 'Table');
    $action->addPhoto($user);

    return true;
  }

  function setAvatar()
  {
    $user      =& JFactory::getUser();
    $media_ids =& JRequest::getVar('media_ids', array(), 'POST', 'array');

    JArrayHelper::toInteger($media_ids);

    if (!count($media_ids))
    {
      $this->setError(JText::_('MY_GALLERY_TASK_SETAVATAR_ERROR_NOT_FOUND'));
      return false;
    }

    $photo =& $this->getTable('photo', 'Table');
    $photo->load($media_ids[0]);

    // Check if photo is valid
    if (count($photo->_errors) || $photo->user_id != $user->id)
    {
      $this->setError(JText::_('MY_GALLERY_TASK_SETAVATAR_ERROR_NOT_FOUND'));
      return false;
    }

    // Unset main photo
    $query = ' UPDATE #__lovefactory_photos'
           . ' SET is_main = 0'
           . ' WHERE is_main = 1'
           . ' AND user_id = ' . $user->id;
    $this->_db->setQuery($query);
    $this->_db->query();

    // Set new main photo
    $photo->is_main = 1;
    $photo->store();

    // Update profile
    $profie =& $this->getTable('profile', 'Table');

    $profie->load($user->id);
    $profie->main_photo = $photo->filename;

    $profie->store();

    $this->_main_id = $photo->id;

    return true;
  }

  function removeAvatar()
  {
    $photo_id =& JRequest::getVar('photo_id', 0, 'POST', 'integer');
    $user     =& JFactory::getUser();

    if (!$photo_id || $user->guest)
    {
      return false;
    }

    $photo =& $this->getTable('photo', 'Table');
    $photo->load($photo_id);

    if ($photo->user_id != $user->id || !$photo->is_main)
    {
      return false;
    }

    $photo->is_main = 0;
    $photo->store();

    // Update profile
    $profie =& $this->getTable('profile', 'Table');

    $profie->load($user->id);
    $profie->main_photo = '';

    $profie->store();

    return true;
  }

  function addGravatar()
  {
    $user =& JFactory::getUser();
    $settings = new LoveFactorySettings();

    if (!$settings->enable_gravatar_integration)
    {
      $this->setError(JText::_('MY_GALLERY_TASK_ADDGRAVATAR_ERROR_NOT_ENABLED'));
      $this->_error_code = 3;
      return false;
    }

    $total   =& $this->getTotalItems('photos', $user);
    $allowed =& $this->getMaximumAllowedItems('photos', $user);

    // Check if user can upload
    if (-1 != $allowed && $total >= $allowed)
    {
      $this->setError(JText::_('MY_GALLERY_TASK_ADDGRAVATAR_ERROR_UPGRADE'));
      $this->_error_code = 2;
      return false;
    }

    $types  =  array('public' => 0, 'friends' => 1, 'private' => 2);
    $date   =& JFactory::getDate();
    $type   =& JRequest::getVar('type', '', 'POST', 'string');
    $status =  $types[$type];
    $table  =& $this->getTable('photo', 'Table');

    /* @var $table TablePhoto */
    $table->user_id    = $user->id;
    $table->filename   = 'gravatar';
    $table->date_added = $date->toMySql();
    $table->status     = $status;
    $table->ordering   = $table->getNextOrder('user_id = ' . $user->id . ' AND status = ' . $status);

    if (!$table->store())
    {
      return false;
    }

    // JSON response
    $types = array(0 => 'public', 1 => 'friends', 2 => 'private');
    $size = max(array($settings->thumbnail_max_height, $settings->thumbnail_max_width));
    $this->_thumbnail_path = 'http://www.gravatar.com/avatar/'.md5(strtolower(trim($user->email))).'?s='.$size.'.jpg';
    $this->_photo_id       = $table->id;
    $this->_type           = $type;

    return true;
  }

  // Video
  function uploadVideoThumbnail()
  {
    // Get uploading user
    $user = $this->getUploadUser();

	  if ($user->guest)
	  {
	    $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_LOGIN'));
	    $this->_error_code = 1;
	    return false;
	  }

    $photo = JRequest::getVar('photo', array(), 'FILES', 'array');

    if (!count($photo))
    {
      $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_TOO_LARGE'));
      $this->_error_code = 3;
      return false;
    }

    // File size over the limit
    if ($photo['size'] == 0)
    {
      $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_SIZE_EXCEEDED'));
      $this->_error_code = 4;
      return false;
    }

    // Not an image
    $image = getimagesize($photo["tmp_name"]);
    if (false === $image)
    {
      $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_NOT_IMAGE'));
      $this->_error_code = 5;
      return false;
    }

    // Not a valid image
    if (!in_array($image['mime'], array('image/jpeg', 'image/gif', 'image/png')))
    {
      $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_NOT_IMAGE'));
      $this->_error_code = 5;
      return false;
    }

    // All is ok
    jimport('joomla.filesystem.file');

    require_once(JPATH_COMPONENT_SITE.DS.'resizeIMG.php');
    require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'settings.php');

    $extension = strtolower(JFile::getExt($photo['name']));
    $filename  = md5($user->id . time() . rand(0, 1000000)) . '.' . $extension;

    $temp_folder = JPATH_COMPONENT_SITE.DS.'storage'.DS.'registration_photos';

    // Photo upload & resize
    $filepath = JPath::clean($temp_folder.DS.$filename);
    $resize = new RESIZEIMAGE($photo['tmp_name']);
    $resize->resize_scale(90, 50, $filepath);
    $resize->close();

    $this->_filename = $filename;
    $this->_source   = JURI::root().'components/com_lovefactory/storage/registration_photos/'.$filename;

    return true;
  }

  function addVideo()
  {
    $user =& JFactory::getUser();

    $total   =& $this->getTotalItems('videos', $user);
    $allowed =& $this->getMaximumAllowedItems('videos', $user);

    // Check if user can upload
    if (-1 != $allowed && $total >= $allowed)
    {
      $this->setError(JText::_('MY_GALLERY_TASK_UPLOAD_ERROR_UPGRADE'));

      $this->_redirect = false;
      $settings = new LovefactorySettings();
      if ($settings->invalid_membership_action)
      {
        $this->_redirect = true;
      }

      return false;
    }

    // Check if user folder exists
    $this->checkUserFolder($user->id);

    // Prepare the status
    $types  =  $this->getTypes();
    $type   =& JRequest::getVar('type', 'public', 'POST', 'string');
    JRequest::setVar('status', $types[$type], 'POST');

    // Prepare the embed code
    $settings = new LovefactorySettings();
    $original = JRequest::getVar('code', '', 'POST', 'string', 2);
    $tags     = explode(',', $settings->videos_embed_allowed_html);
    $stripped = strip_tags($original, '<' . implode('><', $tags) . '>');

    // Get new video
    $video =& $this->getTable('Video');

    $video->bind(JRequest::get('post'));

    $video->user_id = $user->id;
    $video->code    = $stripped;

    // Check for errors
    if (!$video->check())
    {
      $this->setError($video->getError());
      return false;
    }

    // Store the new video
    if (!$video->store())
    {
      $this->setError($video->getError());
      return false;
    }

    $this->setItem($video);

    return true;
  }

  // Helpers
  function getTotalItems($type, $user)
  {
    $query = ' SELECT COUNT(i.id)'
           . ' FROM #__lovefactory_' . $type . ' i'
           . ' WHERE i.user_id = ' . $user->id;
    $this->_db->setQuery($query);

    return (int)$this->_db->loadResult();
  }

  function getMaximumAllowedItems($type, $user)
  {
    $query = ' SELECT m.max_' . $type
           . ' FROM #__lovefactory_profiles p'
           . ' LEFT JOIN #__lovefactory_memberships_sold m ON m.id = p.membership_sold_id'
           . ' WHERE p.user_id = ' . $user->id;
    $this->_db->setQuery($query);

    return (int)$this->_db->loadResult();
  }

  function getUploadUser()
  {
    $session  =& JRequest::getVar('PHPSESSID', '', 'POST', 'string');

    $query = ' SELECT userid'
	         . ' FROM #__session'
	         . ' WHERE session_id = "' . $this->_db->getEscaped($session) . '"';
	  $this->_db->setQuery($query);

	  $user =& JFactory::getUser($this->_db->loadResult());

	  return $user;
  }

  function getTypes()
  {
    return array('public' => 0, 'friends' => 1, 'private' => 2);
  }

  function setItem($item)
  {
    $this->_item = $item;
  }

  function getItem()
  {
    return isset($this->_item) ? $this->_item : new stdClass();
  }

  function checkUserFolder($user_id)
  {
    $user_folder = $this->getUserFolder($user_id);

    if (!file_exists($user_folder))
    {
      mkdir($user_folder);
    }
  }

  function getUserFolder($user_id)
  {
    return JPATH_COMPONENT_SITE.DS.'storage'.DS.'photos'.DS.$user_id;
  }
}