<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

class FrontendModelSearch extends JModel
{
  var $_data;
  var $_total;
  var $_pagination;
  var $_limitstart;
  var $_limit;
  var $_fields;
  var $_show;
  var $_type;
  var $_online;

  function __construct()
  {
    parent::__construct();

    require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'settings.php');
    $settings = new LovefactorySettings();

    // Limit start
    $ajax = (isset($_SERVER['HTTP_X_REQUESTED_WITH']) && ($_SERVER['HTTP_X_REQUESTED_WITH'] == 'XMLHttpRequest'));
    $hash = $ajax ? 'GET' : 'COOKIE';
    $name = $ajax ? 'limitstart' : 'lovefactory_search_limitstart';

    $this->_limitstart = JRequest::getVar($name, 0, $hash, 'integer');
    $this->_limit      = JRequest::getVar('limit',      $settings->number_search_results_per_page, 'REQUEST', 'integer');
    $this->_limitstart = ($this->_limit != 0 ? (floor($this->_limitstart / $this->_limit) * $this->_limit) : 0);

    if ($settings->limit_search_results && ($this->_limitstart + $this->_limit) > $settings->limit_search_results)
    {
      $this->_limitstart = 0;
    }

    $this->_show =& JRequest::getVar('show', 0, 'GET', 'integer');

    setcookie('lovefactory_search_limitstart', $this->_limitstart, null, '/');
  }

  // Pagination
  function getData()
  {
    if (!$this->_show)
    {
      return false;
    }

    if (empty($this->_data))
    {
      $query       = $this->_buildQuery();
      $this->_data = $this->getUsers($query, $this->_limitstart, $this->_limit);
    }
    return $this->_data;
  }

  function _buildQuery($total = false)
  {
    $user    =& JFactory::getUser();
    $profile =& JModel::getInstance('profile', 'FrontendModel');

    $where  =  $this->_buildWhereContent();
    $having =  $this->_buildHavingContent();
    $order  =  $this->_buildOrderContent();

    $query = ' SELECT p.user_id, MAX(se.time) AS time'
           . ' FROM #__lovefactory_profiles p'
           . $this->getLeftJoinsQuery()
           . $where
           . ' GROUP BY p.user_id'
           . $having
           . ($total ? '' : $order);

    return $query;
  }

  function _buildWhereContent()
  {
    static $where = null;

    if ($this->_online)
    {
      return '';
    }

    if (is_null($where))
    {
      $session  =& JFactory::getSession();
      $user     =& JFactory::getUser();
      $post     =  $session->get('lovefactory.search.query', array());
      $array    =  array();
      $settings =  new LovefactorySettings();

      $fields =& JModel::getInstance('fields', 'FrontendModel');
      $fields->getFieldsForPage($this->_type . 'search');

      foreach ($fields->_fields as $field)
      {
        /* @var field TableField */
        $condition = $field->getSearchConditionQuery($post, 'where');

        if ($condition)
        {
          $array[] = $condition;

          if (25 == $field->type_id)
          {
            $this->_relationshipLeftJoin = true;
          }

          if (24 == $field->type_id)
          {
            $this->_distanceLeftJoin = true;
          }
        }
      }

      // Not offline profiles or only for friends and visiting user is not friend
      if ($settings->profile_status_change)
      {
        $array[] = ' (p.online = 0 OR (p.online = 1 AND IF (f.id IS NOT NULL AND f.pending = 0, 1, 0) = 1))';
      }

      // Not my own profile
      if (!$this->_online)
      {
        $array[] = ' p.user_id <> ' . $user->id;
      }

      // Search through banned profiles?
      if ($settings->hide_banned_profiles)
      {
        $array[] = ' p.banned = 0';
      }

      // Users not blocked
      $array[] = ' u.block = 0';

      // Search through ignored profiles?
      if ($settings->hide_ignored_profiles)
      {
        $array[] = ' b.id IS NULL';
      }

      $where = (count($array)) ? ' WHERE ' . implode(' AND ', $array) : '';
    }

    return $where;
  }

  function _buildHavingContent()
  {
    static $having = null;

    if (is_null($having))
    {
      $array    =  array();
      $session  =& JFactory::getSession();
      $post     =  $session->get('lovefactory.search.query', array());

      $fields =& JModel::getInstance('fields', 'FrontendModel');
      $fields->getFieldsForPage($this->_type . 'search');

      foreach ($fields->_fields as $field)
      {
        /* @var field TableField */
        $condition = $field->getSearchConditionQuery($post, 'having');

        if ($condition)
        {
          $array[] = $condition;

          if (13 == $field->type_id)
          {
            $this->_sessionLeftJoin = true;
          }
        }
      }

      // If searching for online profiles
      if ($this->_online)
      {
        $this->_sessionLeftJoin = 1;
        $array[] = ' COUNT(se.userid) <> 0';
      }

      $having = (count($array)) ? ' HAVING ' . implode(' AND ', $array) : '';
    }

    return $having;
  }

  function _buildOrderContent()
  {
    static $order = null;

    if (is_null($order))
    {
      $settings = new LovefactorySettings();

      if ($settings->sort_by_membership)
      {
        $order = ' ORDER BY m.ordering DESC,';
      }
      else
      {
        $order = ' ORDER BY';
      }

      $session    =& JFactory::getSession();
      $sort_by    =  $session->get('lovefactory.search.sortby',    $settings->results_default_sort_by);
      $sort_order =  $session->get('lovefactory.search.sortorder', $settings->results_default_sort_order);

      $sort_order = $sort_order == 1 ? 'ASC' : 'DESC';

      switch ($sort_by)
      {
        case 2: // Username
          $order .= ' u.username';
        break;

        case 3: // Age
          $order .= ' ' . $this->getBirthdateField();
          $sort_order = $sort_order == 'ASC' ? 'DESC' : 'ASC';
        break;

        case 4: // Friends
          $order .= ' p.friends';
        break;

        case 5: // Photos
          $order .= ' p.photos';
        break;

        case 6: // Profle rating
          $order .= ' p.rating';
        break;

        default: // Last seen on
          $date =& JFactory::getDate();
          $diff =  $date->_date - time();

          $order .= ' IF (time IS NULL, UNIX_TIMESTAMP(u.lastvisitDate), time + ' . $diff . ')';

          $sort_order = $sort_order == 'ASC' ? 'DESC' : 'ASC';
        break;
      }

      $order .= ' ' . $sort_order;
    }

    return $order;
  }

  function getTotal()
  {
    if (!$this->_show)
    {
      return false;
    }

    if (empty($this->_total))
    {
      $query        = $this->_buildQuery(true);

      $this->_db->setQuery($query);
      $this->_db->query();

      $this->_total = $this->_db->getNumRows();
    }
    return $this->_total;
  }

  function getPagination()
  {
    if (empty($this->_pagination))
    {
      jimport('joomla.html.pagination');

      $app    =& JFactory::getApplication();
      $router =& $app->getRouter();
      $router->setVar('format', 'raw');

      $total = $this->getTotal();

      $settings = new LovefactorySettings();
      if ($settings->limit_search_results)
      {
        $total = $settings->limit_search_results < $total ? $settings->limit_search_results : $total;
      }

      $this->_pagination = new JPagination($total, $this->_limitstart, $this->_limit);
    }

    return $this->_pagination;
  }

  function getUsers($query, $limitstart, $limit)
  {
    $this->_db->setQuery($query, $limitstart, $limit);
    $this->_data = $this->_db->loadAssocList('user_id');

    if (!count($this->_data))
    {
      return null;
    }

    $order = $this->_buildOrderContent();

    $user    =& JFactory::getUser();
    $profile =& JModel::getInstance('profile', 'FrontendModel');
    $zones = $this->getZonesForResult();
    $extra = $profile->getExtraFields($zones['types']);

    $query = $profile->getMainSelectQuery($extra) . ','
           . '   m.id AS membership_class_id, IF(p.user_id = ' . $user->id . ', 1, 0) AS my_profile,'
           . '   fr.id AS friendship_id, fr.pending AS friendship_pending,'
           . '   IF (fr.sender_id = ' . $user->id . ', 1, 0) AS friendship_my_request'
           . ' FROM #__lovefactory_profiles p'
           . $profile->getLeftJoinsQuery($extra, 'search')
           . ' LEFT JOIN #__lovefactory_friends fr ON ((fr.sender_id = '.$user->id.' AND fr.receiver_id = p.user_id) OR (fr.receiver_id = '.$user->id.' AND fr.sender_id = p.user_id))'
           . ' WHERE p.user_id IN (' . implode(',', array_keys($this->_data)) . ')'
           . ' GROUP BY p.user_id'
           . $order;
    $this->_db->setQuery($query);

    return $this->_db->loadObjectList();
  }

  // View
  function getZones()
  {
    static $zones = null;

    if (!is_null($zones))
    {
      return $zones;
    }

    $model =& JModel::getInstance('zones', 'FrontendModel');
    $zones =  $model->getZonesForDisplayForPage($this->_type . 'search');

    return $zones;
  }

  function getZonesForResult()
  {
    static $zones = null;

    if (!is_null($zones))
    {
      return $zones;
    }

    $model =& JModel::getInstance('zones', 'FrontendModel');
    $zones =  $model->getZonesForDisplayForPage('result');

    return $zones;
  }

  function getResultZones()
  {
    static $zones = null;

    if (!is_null($zones))
    {
      return $zones;
    }

    $model =& JModel::getInstance('zones', 'FrontendModel');
    $zones =  $model->getZonesForDisplayForPage('result');

    return $zones;
  }

  function getDefaults()
  {
    if (!$this->_show)
    {
      return false;
    }

    $session =& JFactory::getSession();
    $post = $session->get('lovefactory.search.query', array());

    if (empty($this->_fields))
    {
      $fields =& JModel::getInstance('fields', 'FrontendModel');
      $fields->getFieldsForPage($this->_type . 'search');

      $this->_fields = $fields->_fields;
    }

    $array = array();

    foreach ($this->_fields as $field)
    {
      $defaults = $field->getDefaultValues();
      $value    = @$post['field_' . $field->id];

      switch ($field->type_id)
      {
        case 1:  // Text input
        case 2:  // Text area
        case 12: // Username
        case 24: // Distance
          $array['field_' . $field->id] = $value;
          break;

        case 3: // Drop down (single select)
          $array['field_' . $field->id] = isset($defaults[$value]) ? $defaults[$value] : '';
          break;

        case 4:  // Drop down (multiple select)
        case 5:  // Checkboxes
        case 13: // Online users
        case 14: // With pictures
        case 25:
          $values = array();

          if ($value)
          {
            if (in_array($field->type_id, array(5, 13, 14, 25)))
            {
              $value = array_keys($value);
            }

            foreach ($value as $val)
            {
              if (isset($defaults[$val]))
              {
                $values[] = $defaults[$val];
              }
            }
          }

          $array['field_' . $field->id] = count($values) ? '*|*'.implode('*|*', $values).'*|*' : '';
          break;

          // Radio buttons
        case 6:
          $array['field_' . $field->id] = isset($defaults[$value]) ? $defaults[$value] : '';
          break;

          // Date birth
        case 7:
          $array['field_' . $field->id] = array('starting' => $post['field_' . $field->id]['starting'],
                                                'ending'   => $post['field_' . $field->id]['ending']);
          break;

          // Sex
        case 10:
          $array['sex'] = isset($defaults[$value]) ? $defaults[$value] : '';
          break;

          // Looking
        case 11:
          if (is_array($value))
          {
            $values = array();

            if (in_array($field->getParam('mode'), array(4)))
            {
              $value = array_keys($value);
            }

            foreach ($value as $val)
            {
              if (isset($defaults[$val]))
              {
                $values[] = $defaults[$val];
              }
            }

            $array['looking'] = count($values) ? '*|*'.implode('*|*', $values).'*|*' : '';
          }
          else
          {
            $array['looking'] = isset($defaults[$value]) ? $defaults[$value] : '';
          }
          break;
      }
    }

    return (object)$array;
  }

  function getShowForm()
  {
    if (!$this->_show)
    {
      return 1;
    }

    return JRequest::getVar('lovefactory_show_search_form', 1, 'COOKIE', 'integer');
  }

  function getPageZones()
  {
    static $zones = null;

    if (is_null($zones))
    {
      $zones_search = $this->getResultZones();
      $zones_result = $this->getZones();

      $zones = array_unique(array_merge($zones_search['types'], $zones_result['types']));
    }

    return $zones;
  }

  function getAdsense()
  {
    $query = ' SELECT a.*'
           . ' FROM #__lovefactory_adsense a';
    $this->_db->setQuery($query);

    return $this->_db->loadObjectList();
  }

  // Tasks
  function updateSearchQuery()
  {
    $session =& JFactory::getSession();
    $session->set('lovefactory.search.query', $_POST);
  }

  function updateSortQuery()
  {
    $sort_by    =& JRequest::getVar('sort_by',    -1, 'REQUEST', 'integer');
    $sort_order =& JRequest::getVar('sort_order', -1, 'REQUEST', 'integer');

    $session =& JFactory::getSession();

    if ($sort_by != -1)
    {
      $session->set('lovefactory.search.sortby', $sort_by);
    }

    if ($sort_order != -1)
    {
      $session->set('lovefactory.search.sortorder', $sort_order);
    }
  }

  // Helpers
  function getBirthdateField()
  {
    $query = ' SELECT id'
           . ' FROM #__lovefactory_fields'
           . ' WHERE type_id = 7'
           . ' LIMIT 0,1';
    $this->_db->setQuery($query);

    $field = $this->_db->loadResult();

    return !is_null($field) ? 'p.field_' . $field : false;
  }

  function getLeftJoinsQuery()
  {
    $user     =& JFactory::getUser();
    $settings =  new LovefactorySettings();

    $output = ' LEFT JOIN #__users u ON u.id = p.user_id'
            . ' LEFT JOIN #__lovefactory_friends f ON ((f.sender_id = p.user_id AND f.receiver_id = ' . $user->id . ') OR (f.receiver_id = p.user_id AND f.sender_id = ' . $user->id . '))';

    // Left Join: Membership (if ordering by membership first)
    if ($settings->sort_by_membership)
    {
      $output .= ' LEFT JOIN #__lovefactory_memberships_sold s ON s.id = p.membership_sold_id'
	            .  ' LEFT JOIN #__lovefactory_memberships m ON m.id = s.membership_id';
    }

    // Left Join: Session
    // - if ordering by last seen
    // - if online members
    $session =& JFactory::getSession();
    $sort_by =  $session->get('lovefactory.search.sortby', 1);

    if (1 || 1 == $sort_by || (isset($this->_sessionLeftJoin) && $this->_sessionLeftJoin))
    {
	    $output .= ' LEFT JOIN #__session se ON se.userid = p.user_id AND se.client_id = 0';
    }

    // Left Join: Relationship
    if (isset($this->_relationshipLeftJoin) && $this->_relationshipLeftJoin)
    {
      $output .= ' LEFT JOIN #__users rel_user ON rel_user.id = p.relationship'
              .  ' LEFT JOIN #__lovefactory_profiles rel_profile ON rel_profile.user_id = p.relationship';
    }

	  // Distance || Google Maps location
	  if (isset($this->_distanceLeftJoin) && $this->_distanceLeftJoin)
	  {
	    $output .= ' LEFT JOIN #__lovefactory_geo g ON g.user_id = p.user_id';
	  }

	  // Blacklist
	  if ($settings->hide_ignored_profiles)
	  {
	    $output .= ' LEFT JOIN #__lovefactory_blacklist b ON (b.sender_id = ' . $user->id . ' AND b.receiver_id = p.user_id)';
	  }

	  return $output;
  }
}